package ru.voskhod.smev.client.api.services.template;

import ru.voskhod.smev.client.api.types.exception.SMEVException;
import ru.voskhod.smev.client.api.types.message.SMEVMessage;
import ru.voskhod.smev.client.api.types.message.system.SMEVMetadata;
import ru.voskhod.smev.client.api.types.message.system.processing.QueryInformation;

import java.util.Map;


/**
 * Шаблон взаимодействия со СМЭВ
 */
public interface WSTemplate {

    /**
     * Получение сообщения (любого типа - запрос/ответ/...) от СМЭВ.
     *
     * @param query фильтр сообщений
     * @return полученное сообщение
     * @throws SMEVException в случае любой ошибки
     */
    SMEVMessage get(QueryInformation query) throws SMEVException;

    /**
     * Отправка сообщение в СМЭВ
     *
     * @param message сообщение для отправки
     * @return метаданные отправленного сообщения
     * @throws SMEVException в случае любой ошибки
     */
    SMEVMetadata send(SMEVMessage message) throws SMEVException;

    /**
     * Подтверждает получение сообщения
     *
     * @param metadata метаданные полученного сообщения
     * @param accepted true - сообщение принято для обработки, false - сообщение отклонено
     * @throws SMEVException в случае любой ошибки
     */
    void ack(SMEVMetadata metadata, boolean accepted) throws SMEVException;

    /**
     * Получает статистику по JMS-очередям текущего клиента
     *
     * @param nodeId идентификатор узла СМЭВ
     * @return количество сообщений в каждой очереди
     * @throws SMEVException в случае любой ошибки
     */
    public Map<String, Long> getIncomingQueueStatistics(String nodeId) throws SMEVException;
}
