package ru.voskhod.smev.client.api.factory.loaders;

import ru.voskhod.smev.client.api.services.identification.IdentityService;
import ru.voskhod.smev.client.api.services.messaging.ExceptionMapper;
import ru.voskhod.smev.client.api.services.messaging.MessageMapper;
import ru.voskhod.smev.client.api.services.signature.PostValidationCallback;
import ru.voskhod.smev.client.api.services.signature.Signer;
import ru.voskhod.smev.client.api.services.template.LargeAttachmentTemplate;
import ru.voskhod.smev.client.api.services.template.MTOMAttachmentTemplate;
import ru.voskhod.smev.client.api.services.template.Template;
import ru.voskhod.smev.client.api.services.template.WSTemplate;
import ru.voskhod.smev.client.api.services.template.configuration.*;
import ru.voskhod.smev.client.api.services.transport.GeoMessageTransport;
import ru.voskhod.smev.client.api.services.transport.LargeAttachmentTransport;
import ru.voskhod.smev.client.api.services.transport.MessageTransport;
import ru.voskhod.smev.client.api.services.transport.configuration.LargeAttachmentTransportConfiguration;
import ru.voskhod.smev.client.api.services.validation.Validator;
import ru.voskhod.smev.client.api.types.exception.SMEVRuntimeException;

/**
 * The interface Template loader.
 */
public interface TemplateLoader {
    <T extends WSTemplateConfiguration> WSTemplate getWsTemplate(Signer signer,
                                                                 Validator validator,
                                                                 MessageMapper mapper,
                                                                 MessageTransport transport,
                                                                 ExceptionMapper manager,
                                                                 LargeAttachmentTransport atransport,
                                                                 T config) throws SMEVRuntimeException;

    <T extends WSTemplateConfiguration> WSTemplate getWsTemplate(Signer signer,
                                                                 Validator validator,
                                                                 MessageMapper mapper,
                                                                 MessageTransport transport,
                                                                 ExceptionMapper manager,
                                                                 LargeAttachmentTransport atransport,
                                                                 T config,
                                                                 PostValidationCallback postValidationCallback)
            throws SMEVRuntimeException;

    <T extends GeoTemplateConfiguration> WSTemplate getWsTemplate(Signer signer,
                                                                 Validator validator,
                                                                 MessageMapper mapper,
                                                                 GeoMessageTransport transport,
                                                                 ExceptionMapper manager,
                                                                 LargeAttachmentTransport atransport,
                                                                 T config) throws SMEVRuntimeException;

    <T extends GeoTemplateConfiguration> WSTemplate getWsTemplate(Signer signer,
                                                                  Validator validator,
                                                                  MessageMapper mapper,
                                                                  GeoMessageTransport transport,
                                                                  ExceptionMapper manager,
                                                                  LargeAttachmentTransport atransport,
                                                                  T config,
                                                                  PostValidationCallback postValidationCallback)
            throws SMEVRuntimeException;

    /**
     * Gets large attachments template.
     *
     * @param <T>    the type parameter
     * @param config the config
     * @return the large attachments template
     * @throws SMEVRuntimeException the smev runtime exception
     */
    <T extends LargeAttachmentTemplateConfiguration> LargeAttachmentTemplate getLargeAttachmentsTemplate(T config) throws SMEVRuntimeException;

    /**
     * Gets ws template.
     *
     * @param <T>    the type parameter
     * @param config the config
     * @return the ws template
     * @throws SMEVRuntimeException the smev runtime exception
     */
    <T extends WSTemplateConfiguration> WSTemplate getWsTemplate(T config) throws SMEVRuntimeException;

    <T extends LargeAttachmentTemplateConfiguration> LargeAttachmentTemplate getLargeAttachmentsTemplate(Signer signer, IdentityService identificationService, LargeAttachmentTransport largeAttachmentTransport);

    /**
     * Gets large attachment transport template.
     *
     * @param <T>    the type parameter
     * @param config the config
     * @return the large attachment transport template
     * @throws SMEVRuntimeException the smev runtime exception
     */
    <T extends LargeAttachmentTransportConfiguration> LargeAttachmentTransport getLargeAttachmentTransportTemplate(T config) throws SMEVRuntimeException;

    /**
     * Gets mtom template.
     *
     * @param <T>    the type parameter
     * @param config the config
     * @return the mtom template
     * @throws SMEVRuntimeException the smev runtime exception
     */
    <T extends MTOMAttachmentTemplateConfiguration> MTOMAttachmentTemplate getMTOMTemplate(T config) throws SMEVRuntimeException;

    <T extends MTOMAttachmentTemplateConfiguration> MTOMAttachmentTemplate getMTOMTemplate(Signer signer, IdentityService identificationService);

    /**
     * Gets template.
     *
     * @param <T>    the type parameter
     * @param config the config
     * @return the template
     * @throws SMEVRuntimeException the smev runtime exception
     */
    <T extends TemplateConfiguration> Template getTemplate(T config) throws SMEVRuntimeException;

    <T extends GeoTemplateConfiguration> Template getTemplate(T config) throws SMEVRuntimeException;
}
