package ru.voskhod.smev.client.api.admin.impl;

import org.w3c.dom.Element;
import ru.gov.smev.admserv.admserv._1.*;
import ru.voskhod.smev.client.api.admin.Constants;
import ru.voskhod.smev.client.api.services.identification.IdentityService;
import ru.voskhod.smev.client.api.services.signature.Signer;
import ru.voskhod.smev.client.api.types.exception.SMEVProcessingException;
import ru.voskhod.smev.client.api.types.exception.SMEVRuntimeException;

import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;
import javax.xml.namespace.QName;
import javax.xml.ws.BindingProvider;
import javax.xml.ws.Service;
import javax.xml.ws.WebEndpoint;
import javax.xml.ws.WebServiceClient;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.GregorianCalendar;

@WebServiceClient(
        name = "AdministrativeService",
        targetNamespace = Constants.ADMINISTRATIVE_SERVICE_NAMESPACE,
        wsdlLocation = "file:/C:/work/smev3_client/implementations/adminserv/src/main/resources/wsdl/ADMSERV.wsdl"
)
final class EndPointCreator extends Service {
    private final int timeout;

    EndPointCreator(URL wsdlDocumentLocation, int timeout, QName serviceName) {
        super(wsdlDocumentLocation, serviceName);
        this.timeout = timeout;
    }

    @WebEndpoint(name = "AdministrativeServicePort")
    AdministrativeServicePort getAdministrativeServicePort() {
        final AdministrativeServicePort port = super.getPort(Constants.ADMINISTRATIVE_SERVICE_PORT, AdministrativeServicePort.class);
        ((BindingProvider) port).getRequestContext().put("com.sun.xml.ws.connect.timeout", timeout);
        return port;
    }

    static AdministrativeServicePort createPort(String url, int timeout, QName serviceQname) throws MalformedURLException {
        final EndPointCreator endClient = new EndPointCreator(new URL(url), timeout, serviceQname);
        return endClient.getAdministrativeServicePort();
    }

    private static Timestamp createTimestamp(IdentityService identity) throws SMEVRuntimeException {
        final Timestamp timestamp = new Timestamp();
        timestamp.setId(identity.generateUUID());
        GregorianCalendar gregorianCalendar = new GregorianCalendar();
        XMLGregorianCalendar calendar = null;
        try {
            calendar = DatatypeFactory.newInstance().newXMLGregorianCalendar(gregorianCalendar);
        } catch (DatatypeConfigurationException e) {
            throw new SMEVRuntimeException(e.getMessage(),e) ;
        }
        timestamp.setValue(calendar);
        return timestamp;
    }

    static SwitchRequest createRequest(IdentityService identity, Signer signer, TransformerServiceLocal transformer)
            throws SMEVProcessingException, SMEVRuntimeException {
        final SwitchRequest request = new SwitchRequest();
        final Timestamp timestamp = createTimestamp(identity);
        final Element marshal = transformer.marshal(timestamp, null);
        final Element sigDetached = signer.signXMLDSigDetached(marshal, null);
        final XMLDSigSignatureType signature = new XMLDSigSignatureType();
        signature.setAny(sigDetached);
        request.setTimestamp(timestamp);
        request.setCallerInformationSystemSignature(signature);
        return request;
    }

    static CurrentNodeRequest createNodeRequest(IdentityService identity, Signer signer, TransformerServiceLocal transformer)
            throws SMEVProcessingException, SMEVRuntimeException {
        final CurrentNodeRequest request = new CurrentNodeRequest();
        final Timestamp timestamp = createTimestamp(identity);
        final Element marshal = transformer.marshal(timestamp, null);
        final Element sigDetached = signer.signXMLDSigDetached(marshal, null);
        final XMLDSigSignatureType signature = new XMLDSigSignatureType();
        signature.setAny(sigDetached);
        request.setTimestamp(timestamp);
        request.setCallerInformationSystemSignature(signature);
        return request;
    }
}
