package ru.voskhod.smev.client.api.admin.impl;


import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import ru.voskhod.smev.client.api.types.exception.SMEVRuntimeException;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;
import javax.xml.transform.*;
import javax.xml.transform.dom.DOMResult;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import java.io.StringWriter;

final class TransformerServiceLocalImpl implements TransformerServiceLocal {
    private static final Logger logger = LoggerFactory.getLogger(TransformerServiceLocalImpl.class);

    static JAXBContext init() {
        try {
            return JAXBContext.newInstance("ru.gov.smev.admserv.admserv._1");
        } catch (JAXBException e) {
            logger.error("ooops. Something went wrong", e);
            return null;
        }
    }

    //
    private final static JAXBContext jxbcontext = init();
    private static ThreadLocal<Marshaller> jaxbMarshaller = new ThreadLocal<Marshaller>() {
        @Override
        protected Marshaller initialValue() {
            try {
                return jxbcontext.createMarshaller();
            } catch (JAXBException e) {
                throw new RuntimeException(e);
            }
        }
    };
    private static ThreadLocal<Unmarshaller> jaxbUnmarshaller = new ThreadLocal<Unmarshaller>() {
        @Override
        protected Unmarshaller initialValue() {
            try {
                return jxbcontext.createUnmarshaller();
            } catch (JAXBException e) {
                throw new RuntimeException(e);
            }
        }
    };

    private static ThreadLocal<Transformer> nullTransformer = new ThreadLocal<Transformer>() {
        @Override
        protected Transformer initialValue() {
            try {
                TransformerFactory factory = TransformerFactory.newInstance();
                return factory.newTransformer();
            } catch (TransformerConfigurationException | TransformerFactoryConfigurationError e) {
                throw new RuntimeException(e);
            }
        }
    };

    public Element marshal(Object arg, String error) throws SMEVRuntimeException {
        DOMResult domResult = new DOMResult();
        try {
            try {
                jaxbMarshaller.get().marshal(arg, domResult);
            } catch (JAXBException e) {
                logger.error("Can not convert JAXB object into DOM.", e);
                throw new SMEVRuntimeException("Can not convert JAXB object into DOM.");
            }
            return ((Document) domResult.getNode()).getDocumentElement();
        } catch (IllegalArgumentException e) {
            logger.debug("Illegal content in business envelope; could not be processed by JAXB.");
            throw new SMEVRuntimeException("Illegal content in business envelope; could not be processed by JAXB.");
        }
    }

    public Object unmarshal(Element arg) throws SMEVRuntimeException {
        // XML-содержимое сообщения.
        try {
            return jaxbUnmarshaller.get().unmarshal(arg);
        } catch (JAXBException e) {
            logger.error("Can not unmarshal user provided data.", e);
            throw new SMEVRuntimeException("Can not unmarshal user provided data.");
        }
    }

    public String marshal2string(Object arg) throws SMEVRuntimeException {
        Element asDOM = marshal(arg, "qqq");
        return marshal2string(asDOM);
    }

    public String marshal2string(Element arg) throws SMEVRuntimeException {
        StringWriter swr = new StringWriter();
        Result result = new StreamResult(swr);
        Source source = new DOMSource(arg);
        try {
            nullTransformer.get().transform(source, result);
        } catch (TransformerException e) {
            logger.warn("Can not marshal a DOM element into String.", e);
            throw new SMEVRuntimeException("Can not marshal a DOM element into String.");
        }

        return swr.getBuffer().toString();
    }
}
