package ru.voskhod.smev.client.api.identification.impl;

import com.fasterxml.uuid.EthernetAddress;
import com.fasterxml.uuid.UUIDTimer;
import com.fasterxml.uuid.impl.TimeBasedGenerator;
import ru.voskhod.smev.client.api.services.identification.IdentityService;
import ru.voskhod.smev.client.api.util.Utils;

import java.io.IOException;
import java.util.Random;
import java.util.UUID;

/**
 * The type Identification service.
 */
public class IdentificationServiceImpl implements IdentityService {
    /**
     * The constant ID_CONSTANT.
     */
    public static final String ID_CONSTANT = "__ATT_ID_SMEV_C_AUTOGEN__";

    private static final TimeBasedGenerator uuidGenerator;

    static {
        Random random = new Random(System.currentTimeMillis());
        UUIDTimer timer;
        try {
            timer = new UUIDTimer(random, null);
        } catch (IOException e) {
            // Will never be thrown.
            throw new RuntimeException(e);
        }
        EthernetAddress addr = EthernetAddress.fromInterface();

        uuidGenerator = new TimeBasedGenerator(addr, timer);
    }

    @Override
    public String generateUUID() {
        return uuidGenerator.generate().toString();
    }

    @Override
    public String generateAttachmentId(String mimeType, int attachmentNumber) {
        String ext = Utils.guessFileExtension(mimeType);
        return ID_CONSTANT + (attachmentNumber) + ext;
    }

    @Override
    public UUID generateAttachmentUUID() {
        return UUID.randomUUID();
    }
}
