package ru.voskhod.smev.client.api.message.impl;


import ru.voskhod.smev.client.api.services.messaging.ExceptionMapper;
import ru.voskhod.smev.client.api.types.exception.SMEVProcessingException;
import ru.voskhod.smev.client.api.types.exception.SMEVRuntimeException;
import ru.voskhod.smev.client.api.types.exception.processing.*;
import ru.voskhod.smev.server.autogenerated.types.basic.SmevFault;
import ru.voskhod.smev.server.autogenerated.types.basic.Void;
import ru.voskhod.smev.server.autogenerated.types.fault.*;
import ru.voskhod.smev.server.autogenerated.*;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;


/**
 * The type Exception mapper.
 */
public class ExceptionMapperImpl implements ExceptionMapper<SmevFault> {
    /**
     * The constant SMEV_VOID_CODE.
     */
    public static final String SMEV_VOID_CODE = "-1";

    @Override
    public SMEVProcessingException convert(Throwable exception) throws SMEVRuntimeException {
        if (exception instanceof Exception) {
            try {
                throw (Exception) exception;
            } catch (SMEVProcessingException ex) {
                return ex;
            } catch (EndOfLifeException ex) {
                return new SMEVRoutingException(ex.getMessage(), getCode(ex.getFaultInfo()), new SMEVRoutingException.EndOfLifeException(ex.getMessage(), getDescription(ex.getFaultInfo()), ex));
            } catch (ru.voskhod.smev.server.autogenerated.SMEVFailureException ex) {
                return new ru.voskhod.smev.client.api.types.exception.processing.SMEVFailureException(ex.getMessage(), getCode(ex.getFaultInfo()), getDescription(ex.getFaultInfo()), ex);
            } catch (AccessDeniedException ex) {
                return new SMEVRoutingException(ex.getMessage(), getCode(ex.getFaultInfo()), new SMEVRoutingException.AccessDeniedException(ex.getMessage(), getDescription(ex.getFaultInfo()), ex));
            } catch (StaleMessageIdException ex) {
                return new SMEVMessageIdentityException(ex.getMessage(), getCode(ex.getFaultInfo()), new SMEVMessageIdentityException.StaleMessageIdException(ex.getMessage(), getDescription(ex.getFaultInfo()), ex));
            } catch (InvalidContentException ex) {
                return new SMEVMessageContentException(ex.getMessage(), getCode(ex.getFaultInfo()), new SMEVMessageContentException.InvalidContentException(ex.getMessage(), convert(ex.getFaultInfo()), getDescription(ex.getFaultInfo()), ex));
            } catch (QuoteLimitExceededException ex) {
                return new SMEVAttachmentsException(ex.getMessage(), getCode(ex.getFaultInfo()), new SMEVAttachmentsException.QuoteLimitExceededException(ex.getMessage(), convert(ex.getFaultInfo()), getDescription(ex.getFaultInfo()), ex));
            } catch (UnknownMessageTypeException ex) {
                return new SMEVMessageContentException(ex.getMessage(), getCode(ex.getFaultInfo()), new SMEVMessageContentException.UnknownMessageTypeException(ex.getMessage(), getDescription(ex.getFaultInfo()), ex));
            } catch (DestinationOverflowException ex) {
                return new SMEVResourceException(ex.getMessage(), getCode(ex.getFaultInfo()), new SMEVResourceException.DestinationOverflowException(ex.getMessage(), convert(ex.getFaultInfo()), getDescription(ex.getFaultInfo()), ex));
            } catch (RecipientIsNotFoundException ex) {
                return new SMEVRoutingException(ex.getMessage(), getCode(ex.getFaultInfo()), new SMEVRoutingException.RecipientIsNotFoundException(ex.getMessage(), getDescription(ex.getFaultInfo()), ex));
            } catch (MessageIsAlreadySentException ex) {
                return new SMEVMessageIdentityException(ex.getMessage(), getCode(ex.getFaultInfo()), new SMEVMessageIdentityException.MessageIsAlreadySentException(ex.getMessage(), getDescription(ex.getFaultInfo()), ex));
            } catch (SenderIsNotRegisteredException ex) {
                return new SMEVRoutingException(ex.getMessage(), getCode(ex.getFaultInfo()), new SMEVRoutingException.SenderIsNotRegisteredException(ex.getMessage(), getDescription(ex.getFaultInfo()), ex));
            } catch (TransactionCodeInvalidException ex) {
                return new SMEVMessageIdentityException(ex.getMessage(), getCode(ex.getFaultInfo()), new SMEVMessageIdentityException.TransactionCodeInvalidException(ex.getMessage(), getDescription(ex.getFaultInfo()), ex));
            } catch (InvalidMessageIdFormatException ex) {
                return new SMEVMessageIdentityException(ex.getMessage(), getCode(ex.getFaultInfo()), new SMEVMessageIdentityException.InvalidMessageIdFormatException(ex.getMessage(), getDescription(ex.getFaultInfo()), ex));
            } catch (TargetMessageIsNotFoundException ex) {
                return new SMEVRoutingException(ex.getMessage(), getCode(ex.getFaultInfo()), new SMEVRoutingException.TargetMessageIsNotFoundException(ex.getMessage(), getDescription(ex.getFaultInfo()), ex));
            } catch (SignatureVerificationFaultException ex) {
                return new SMEVSignatureException(ex.getMessage(), getCode(ex.getFaultInfo()), new SMEVSignatureException.SignatureVerificationFaultException(ex.getMessage(), convert(ex.getFaultInfo()), getDescription(ex.getFaultInfo()), ex));
            } catch (AttachmentSizeLimitExceededException ex) {
                return new SMEVAttachmentsException(ex.getMessage(), getCode(ex.getFaultInfo()), new SMEVAttachmentsException.SizeLimitExceededException(ex.getMessage(), convert(ex.getFaultInfo()), getDescription(ex.getFaultInfo()), ex));
            } catch (IncorrectResponseContentTypeException ex) {
                return new SMEVMessageContentException(ex.getMessage(), getCode(ex.getFaultInfo()), new SMEVMessageContentException.IncorrectResponseContentTypeException(ex.getMessage(), getDescription(ex.getFaultInfo()), ex));
            } catch (BusinessDataTypeIsNotSupportedException ex) {
                return new SMEVMessageContentException(ex.getMessage(), getCode(ex.getFaultInfo()), new SMEVMessageContentException.BusinessDataTypeIsNotSupportedException(ex.getMessage(), convert(ex.getFaultInfo()), getDescription(ex.getFaultInfo()), ex));
            } catch (AttachmentContentMiscoordinationException ex) {
                return new SMEVAttachmentsException(ex.getMessage(), getCode(ex.getFaultInfo()), new SMEVAttachmentsException.MetadataException(ex.getMessage(), getDescription(ex.getFaultInfo()), ex));
            } catch (Exception ex) {
                try {
                    Method methodToFind = null;
                    methodToFind = ex.getClass().getMethod("getFaultInfo", (Class<?>[]) null);
                    if (methodToFind != null) {
                        // Method found. You can invoke the method like
                        Object result = methodToFind.invoke(ex, (Object[]) null);
                        if (result instanceof SmevFault) {
                            return new SMEVEncodedException(((SmevFault) result).getDescription(), ((SmevFault) result).getCode(), ex);
                        }
                    } else
                        throw new SMEVRuntimeException("Unknown exception type. See cause.", ex);
                } catch (Exception ex1) {
                    throw new SMEVRuntimeException("Unknown exception type. See cause.", ex);
                }
            }
        }
        throw new SMEVRuntimeException("Unknown exception type. See cause.", exception);
    }

    private String convert(SignatureVerificationFault faultInfo) {
        if (faultInfo == null)
            return null;
        return faultInfo.getSignatureVerificationFault();
    }

    private SMEVMessageContentException.BusinessDataTypeIsNotSupportedException.BusinessDataTypeIsNotSupported convert(BusinessDataTypeIsNotSupported fault) {
        if (fault != null) {
            return new SMEVMessageContentException.BusinessDataTypeIsNotSupportedException.BusinessDataTypeIsNotSupported(fault.getRootElementLocalName(), fault.getRootElementNamespaceURI());
        } else
            return null;
    }

    private SMEVAttachmentsException.SizeLimitExceededException.SizeLimitInfo convert(AttachmentSizeLimitExceeded fault) {
        if (fault != null) {
            return new SMEVAttachmentsException.SizeLimitExceededException.SizeLimitInfo(fault.getPermittedTotalAttachmentSize(), fault.getRealTotalAttachmentSize());
        } else
            return null;
    }

    private String getCode(Void aVoid) {
        if (aVoid instanceof SmevFault) {
            return ((SmevFault) aVoid).getCode();
        } else {
            return SMEV_VOID_CODE;
        }
    }

    private String getDescription(Void aVoid) {
        if (aVoid instanceof SmevFault) {
            return ((SmevFault) aVoid).getDescription();
        } else {
            return null;
        }
    }

    private SMEVResourceException.DestinationOverflowException.DestinationOverflow convert(DestinationOverflow destinationOverflow) {
        if (destinationOverflow == null)
            return null;
        return new SMEVResourceException.DestinationOverflowException.DestinationOverflow(destinationOverflow.getMessageBrokerAddress(), destinationOverflow.getDestinationName());
    }

    private SMEVAttachmentsException.QuoteLimitExceededException.QuoteLimitExceeded convert(QuoteLimitExceeded quoteLimitExceeded) {
        if (quoteLimitExceeded == null)
            return null;
        return new SMEVAttachmentsException.QuoteLimitExceededException.QuoteLimitExceeded(quoteLimitExceeded.getRemainedTotalQuoteSize(), quoteLimitExceeded.getRealTotalAttachmentSize());
    }

    private SMEVMessageContentException.InvalidContentException.InvalidContent convert(InvalidContent invalidContent) {
        if (invalidContent == null || invalidContent.getValidationError() == null)
            return null;
        List<SMEVMessageContentException.InvalidContentException.InvalidContent.ValidationError> validationErrors = new ArrayList<>();
        for (InvalidContent.ValidationError validationError : invalidContent.getValidationError()) {
            SMEVMessageContentException.InvalidContentException.InvalidContent.ValidationError ve =
                    new SMEVMessageContentException.InvalidContentException.InvalidContent.ValidationError(validationError.getValue(), validationError.getErrorPosition());
            validationErrors.add(ve);
        }
        return new SMEVMessageContentException.InvalidContentException.InvalidContent(validationErrors);
    }

    @Override
    public SMEVProcessingException convert(SmevFault fault) {
        if (fault == null)
            return null;

        if (fault instanceof AttachmentSizeLimitExceeded) {
            return new SMEVAttachmentsException(fault.getDescription(), fault.getCode(), new SMEVAttachmentsException.SizeLimitExceededException(fault.getDescription(), convert((AttachmentSizeLimitExceeded) fault)));
        } else if (fault instanceof BusinessDataTypeIsNotSupported) {
            return new SMEVMessageContentException(fault.getDescription(), fault.getCode(), new SMEVMessageContentException.BusinessDataTypeIsNotSupportedException(fault.getDescription(), convert((BusinessDataTypeIsNotSupported) fault)));
        } else if (fault instanceof DestinationOverflow) {
            return new SMEVResourceException(fault.getDescription(), fault.getCode(), new SMEVResourceException.DestinationOverflowException(fault.getDescription(), convert((DestinationOverflow) fault)));
        } else if (fault instanceof InvalidContent) {
            return new SMEVMessageContentException(fault.getDescription(), fault.getCode(), new SMEVMessageContentException.InvalidContentException(fault.getDescription(), convert((InvalidContent) fault)));
        } else if (fault instanceof QuoteLimitExceeded) {
            return new SMEVAttachmentsException(fault.getDescription(), fault.getCode(), new SMEVAttachmentsException.QuoteLimitExceededException(fault.getDescription(), convert((QuoteLimitExceeded) fault)));
        } else if (fault instanceof SignatureVerificationFault) {
            return new SMEVSignatureException(fault.getDescription(), fault.getCode(), new SMEVSignatureException.SignatureVerificationFaultException(fault.getDescription(), ((SignatureVerificationFault) ((SignatureVerificationFault) fault)).getSignatureVerificationFault()));
        }
        return new SMEVEncodedException(fault.getDescription(), fault.getCode());
    }

}
