package ru.voskhod.smev.client.api.message.impl;

import ru.voskhod.smev.client.api.message.impl.parsers.RequestBusinessMetadataParser;
import ru.voskhod.smev.client.api.message.impl.parsers.ResponseBusinessMetadataParser;
import ru.voskhod.smev.client.api.services.messaging.MessageMapper;
import ru.voskhod.smev.client.api.types.exception.SMEVException;
import ru.voskhod.smev.client.api.types.exception.SMEVRuntimeException;
import ru.voskhod.smev.client.api.types.message.SMEVMessage;
import ru.voskhod.smev.client.api.types.message.attachment.LargeAttachment;
import ru.voskhod.smev.client.api.types.message.attachment.MTOMAttachment;
import ru.voskhod.smev.client.api.types.message.attachment.SMEVAttachment;
import ru.voskhod.smev.client.api.types.message.business.data.*;
import ru.voskhod.smev.client.api.types.message.business.data.request.RequestContent;
import ru.voskhod.smev.client.api.types.message.business.data.response.RejectResponseContent;
import ru.voskhod.smev.client.api.types.message.business.data.response.ResponseContent;
import ru.voskhod.smev.client.api.types.message.business.data.response.StatusResponseContent;
import ru.voskhod.smev.client.api.types.message.business.data.response.UnknownResponseContent;
import ru.voskhod.smev.client.api.types.message.system.processing.ProcessingInformation;
import ru.voskhod.smev.client.api.types.message.system.processing.QueryInformation;
import ru.voskhod.smev.client.api.types.message.system.processing.RequestInformation;
import ru.voskhod.smev.client.api.types.message.system.processing.ResponseInformation;
import ru.voskhod.smev.client.api.types.message.system.SMEVContext;
import ru.voskhod.smev.client.api.types.message.system.SMEVMetadata;
import ru.voskhod.smev.client.api.types.message.system.enums.SMEVStatus;
import ru.voskhod.smev.client.api.util.Utils;
import ru.voskhod.smev.server.autogenerated.types.*;
import ru.voskhod.smev.server.autogenerated.types.basic.*;
import ru.voskhod.smev.server.autogenerated.types.basic.ObjectFactory;
import ru.voskhod.smev.server.autogenerated.types.basic.Void;

import javax.xml.datatype.XMLGregorianCalendar;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.*;

/**
 * The type Message mapper.
 */
public class MessageMapperImpl implements MessageMapper<Void, AckRequest, GetStatusResponse, GetStatusRequest, Object,
        Object, GetResponseResponse, GetResponseRequest, SendResponseResponse, SendResponseRequest,
        GetRequestResponse, GetRequestRequest, SendRequestResponse, SendRequestRequest> {

    /**
     * The constant SIGNATURE_ID_CONSUMER.
     */
    public static final String SIGNATURE_ID_CONSUMER = "SIGNED_BY_CONSUMER";
    /**
     * The constant SIGNATURE_ID_PROVIDER.
     */
    public static final String SIGNATURE_ID_PROVIDER = "SIGNED_BY_PROVIDER";
    /**
     * The constant SIGNATURE_ID_CALLER.
     */
    public static final String SIGNATURE_ID_CALLER = "SIGNED_BY_CALLER";
    private static final ru.voskhod.smev.server.autogenerated.types.ObjectFactory typesFactory = new ru.voskhod.smev.server.autogenerated.types.ObjectFactory();
    private static final ObjectFactory basicFactory = new ObjectFactory();

    private <T> SMEVMessage convert(T response, Class<T> type) throws SMEVException {
        if (type.getClass().equals(GetRequestResponse.class)) {
            return convertGetRequestResponse((GetRequestResponse) response);
        }
        return null;
    }

    @Override
    public SMEVContext convertAckResponse(Void ackResponse) {
        return new SMEVContext(null, null, null, null, null, null, null, null, null, null, null);
    }

    @Override
    public AckRequest createAckRequest(String messageId, boolean accepted) {
        AckRequest ackRequest = typesFactory.createAckRequest();
        AckTargetMessage ackTargetMessage = basicFactory.createAckTargetMessage();
        ackTargetMessage.setId(SIGNATURE_ID_CALLER);
        ackTargetMessage.setAccepted(accepted);
        ackTargetMessage.setValue(messageId);
        ackRequest.setAckTargetMessage(ackTargetMessage);
        return ackRequest;
    }

    @Override
    public SMEVMessage convertStatusResponse(GetStatusResponse getStatusResponse) {
        AsyncProcessingStatus status = getStatusResponse.getSmevAsyncProcessingMessage().getAsyncProcessingStatusData().getAsyncProcessingStatus();
        SMEVContext smevContext = new SMEVContext(null, null, null, null, null, null, status.getStatusDetails(), null, convertStatus(status.getStatusCategory()), null, new ExceptionMapperImpl().convert(status.getSmevFault()));
        SMEVMetadata smevMetadata = new SMEVMetadata(new SMEVMetadata.MessageIdentity(getStatusResponse.getSmevAsyncProcessingMessage().getAsyncProcessingStatusData().getAsyncProcessingStatus().getOriginalMessageId(), null, null), null, null);
        smevMetadata.setSmevContext(smevContext);
        return new SMEVMessage(smevMetadata, null);
    }

    @Override
    public GetStatusRequest createGetStatusRequest() {
        GetStatusRequest getStatusRequest = typesFactory.createGetStatusRequest();
        Timestamp timestamp = basicFactory.createTimestamp();
        timestamp.setId(SIGNATURE_ID_CALLER);
        timestamp.setValue(Utils.getNow());
        getStatusRequest.setTimestamp(timestamp);
        return getStatusRequest;
    }

    @Override
    public Map<String, Long> convertStatisticsResponse(Object getIncomingQueueStatisticsResponse) {
        throw new UnsupportedOperationException();
    }

    @Override
    public Object createGetIncomingQueueStatisticsRequest(String nodeId) {
       throw new UnsupportedOperationException();
    }

    @Override
    public SMEVMessage convertGetResponseResponse(GetResponseResponse getResponseResponse) throws SMEVException {
        GetResponseResponse.ResponseMessage responseMessage = getResponseResponse.getResponseMessage();
        if (responseMessage == null || responseMessage.getResponse() == null) {
            return null;
        }
        Response response = responseMessage.getResponse();

        SMEVMetadata.MessageIdentity messageIdentity = new SMEVMetadata.MessageIdentity(response.getSenderProvidedResponseData().getMessageID(), response.getReferenceMessageID(), response.getOriginalTransactionCode());
        ResponseInformation responseInformation = new ResponseInformation(response.getOriginalMessageId(), response.getSenderProvidedResponseData().getTo());//FIXME: fill nodeId
        SMEVMetadata smevMetadata = new SMEVMetadata(messageIdentity, responseInformation);

        List<SMEVAttachment> attachments = createAttachments(response.getFSAttachmentsList(), response.getSenderProvidedResponseData().getAttachmentHeaderList(), response.getSenderProvidedResponseData().getRefAttachmentHeaderList(), responseMessage.getAttachmentContentList());
        BusinessContent businessData = createBusinessData(response.getSenderProvidedResponseData(), attachments);
        SMEVContext smevContext;

        if (businessData == null) {
            smevContext = createContext(response.getMessageMetadata(), response.getSenderProvidedResponseData().getAsyncProcessingStatus());
        } else {
            smevContext = createContext(response.getMessageMetadata(), null, null, null);
        }


        smevMetadata.setSmevContext(smevContext);
        return new SMEVMessage(smevMetadata, businessData);
    }

    private SMEVContext createContext(MessageMetadata metadata, AsyncProcessingStatus status) {
        SMEVContext.Participant sender = new SMEVContext.Participant(metadata.getSender()!=null?metadata.getSender().getMnemonic():"Not detected", "Not detected");
        SMEVContext.Participant recipient = new SMEVContext.Participant(metadata.getRecipient()!=null?metadata.getRecipient().getMnemonic():"Not detected", "Not detected");
        String destinationName = "Not supported";
        String contentTypeName = "Not supported";
        XMLGregorianCalendar sendingTimestamp = metadata.getSendingTimestamp();
        XMLGregorianCalendar deliveryTimestamp = metadata.getDeliveryTimestamp();
        String interactionType = "Not supported";
        return new SMEVContext(sender, recipient, destinationName, contentTypeName, sendingTimestamp, deliveryTimestamp, status.getStatusDetails(), interactionType, SMEVStatus.REJECTED, null, new ExceptionMapperImpl().convert(status.getSmevFault()));
    }

    private BusinessContent createBusinessData(SenderProvidedResponseData senderProvidedResponseData, List<SMEVAttachment> attachments) throws SMEVRuntimeException {
        if (senderProvidedResponseData.getMessagePrimaryContent() != null) {
            return createResponseBusinessData(senderProvidedResponseData, attachments);
        } else if (senderProvidedResponseData.getRequestStatus() != null) {
            return createInfoBusinessData(senderProvidedResponseData, attachments);
        } else if (senderProvidedResponseData.getAsyncProcessingStatus() != null) {
            return null;
        } else if (senderProvidedResponseData.getRequestRejected() != null && senderProvidedResponseData.getRequestRejected().size() > 0) {
            return createCancelBusinessData(senderProvidedResponseData, attachments);
        } else {
            return createUnknownBusinessData(senderProvidedResponseData, attachments);
        }
    }

    private BusinessContent createResponseBusinessData(SenderProvidedResponseData senderProvidedResponseData, List<SMEVAttachment> attachments) throws SMEVRuntimeException {
        return new ResponseContent(senderProvidedResponseData.getMessagePrimaryContent().getAny(), senderProvidedResponseData.getPersonalSignature() == null ? null : senderProvidedResponseData.getPersonalSignature().getAny(), attachments, null);
    }

    private BusinessContent createCancelBusinessData(SenderProvidedResponseData senderProvidedResponseData, List<SMEVAttachment> attachments) throws SMEVRuntimeException {
        final RejectResponseContent responseContent = new RejectResponseContent(null);
        if (senderProvidedResponseData.getRequestRejected() != null && senderProvidedResponseData.getRequestRejected().size() > 0) {

            final List<SenderProvidedResponseData.RequestRejected> rejected = senderProvidedResponseData.getRequestRejected();

            for (SenderProvidedResponseData.RequestRejected reject : rejected) {
                responseContent.add(getStatusCode(reject.getRejectionReasonCode()),
                        reject.getRejectionReasonDescription());
            }
            return responseContent;
        } else {
            responseContent.add(RejectResponseContent.RejectCode.UNKNOWN_REQUEST_DESCRIPTION, "unknown");
            return responseContent;
        }
    }

    private RejectResponseContent.RejectCode getStatusCode(RejectCode rejectCode) {
        try {
            return RejectResponseContent.RejectCode.fromValue(rejectCode.value());
        } catch (Exception ex) {
            return RejectResponseContent.RejectCode.UNKNOWN_REQUEST_DESCRIPTION;
        }
    }

    private BusinessContent createInfoBusinessData(SenderProvidedResponseData senderProvidedResponseData, List<SMEVAttachment> attachments) throws SMEVRuntimeException {
        if (senderProvidedResponseData.getRequestStatus() != null) {
            SenderProvidedResponseData.RequestStatus requestStatus = senderProvidedResponseData.getRequestStatus();
            return new StatusResponseContent(requestStatus.getStatusCode(), requestStatus.getStatusDescription(), convert(requestStatus.getStatusParameter()), null);
        } else {
            return new StatusResponseContent(-1, "unknown", null, null);
        }
    }

    private Map<String, String> convert(List<SenderProvidedResponseData.RequestStatus.StatusParameter> statusParameters) {
        Map<String, String> parameters = new HashMap<>();
        for (SenderProvidedResponseData.RequestStatus.StatusParameter statusParameter : statusParameters) {
            parameters.put(statusParameter.getKey(), statusParameter.getValue());
        }
        return parameters;
    }

    private BusinessContent createUnknownBusinessData(SenderProvidedResponseData senderProvidedResponseData, List<SMEVAttachment> attachments) throws SMEVRuntimeException {
        return new UnknownResponseContent(null);
    }

    @Override
    public GetResponseRequest createGetResponseRequest(QueryInformation query) {
        GetResponseRequest getResponseRequest = typesFactory.createGetResponseRequest();

        MessageTypeSelector messageTypeSelector = basicFactory.createMessageTypeSelector();
        messageTypeSelector.setId(SIGNATURE_ID_CALLER);
        messageTypeSelector.setTimestamp(Utils.getNow());
        messageTypeSelector.setNamespaceURI(query.getNamespaceURI());
        messageTypeSelector.setRootElementLocalName(query.getRootElementLocalName());
        messageTypeSelector.setNodeID(query.getNodeId());
        getResponseRequest.setMessageTypeSelector(messageTypeSelector);

        return getResponseRequest;
    }

    @Override
    public SMEVMetadata convertSendResponseResponse(SendResponseResponse sendResponseResponse) {
        SMEVMetadata smevMetadata = new SMEVMetadata(new SMEVMetadata.MessageIdentity(sendResponseResponse.getMessageMetadata().getMessageId(), null, null), null, null);
        smevMetadata.setSmevContext(createContext(sendResponseResponse.getMessageMetadata(), null, null, null));
        return smevMetadata;
    }

    @Override
    public SendResponseRequest createSendResponseRequest(SMEVMessage smevMessage) {
        SendResponseRequest sendResponseRequest = typesFactory.createSendResponseRequest();

        sendResponseRequest.setAttachmentContentList(convertMTOMAttachments(smevMessage.getData().getAttachments()));
        sendResponseRequest.setSenderProvidedResponseData(createSenderProvidedResponseData(smevMessage));

        return sendResponseRequest;
    }

    @Override
    public SMEVMessage convertGetRequestResponse(GetRequestResponse getRequestResponse) throws SMEVException {
        GetRequestResponse.RequestMessage requestMessage = getRequestResponse.getRequestMessage();
        if (requestMessage == null) {
            return null;
        }

        SMEVMetadata smevMetadata = null;
        BusinessContent businessData = null;
        SMEVContext smevContext = null;

        if (requestMessage.getRequest() != null) {
            Request request = requestMessage.getRequest();
            smevMetadata = createSMEVMetadata(request.getSenderProvidedRequestData());
            List<SMEVAttachment> attachments = createAttachments(request.getFSAttachmentsList(), request.getSenderProvidedRequestData().getAttachmentHeaderList(), request.getSenderProvidedRequestData().getRefAttachmentHeaderList(), requestMessage.getAttachmentContentList());
            businessData = new RequestContent(request.getSenderProvidedRequestData().getMessagePrimaryContent().getAny(), request.getSenderProvidedRequestData().getPersonalSignature() == null ? null : request.getSenderProvidedRequestData().getPersonalSignature().getAny(), attachments, request.getSenderProvidedRequestData().getBusinessProcessMetadata() == null ? null : request.getSenderProvidedRequestData().getBusinessProcessMetadata().getAny());
            smevContext = createContext(request.getMessageMetadata(), request.getReplyTo(), null, null);
        }
        smevMetadata.setSmevContext(smevContext);
        return new SMEVMessage(smevMetadata, businessData);
    }

    private List<SMEVAttachment> createAttachments(FSAttachmentsList fsAttachmentsList, AttachmentHeaderList attachmentHeaderList, RefAttachmentHeaderList refAttachmentHeaderList, AttachmentContentList attachmentContentList) throws SMEVException {
        List<SMEVAttachment> smevAttachments = new ArrayList<>();
        try {
            smevAttachments.addAll(createLargeAttachments(fsAttachmentsList, refAttachmentHeaderList));
        } catch (URISyntaxException | MalformedURLException e) {
            throw new SMEVRuntimeException("Bad file name!", e);
        }
        smevAttachments.addAll(createMtomAttachments(attachmentHeaderList, attachmentContentList));
        return smevAttachments;
    }

    private List<MTOMAttachment> createMtomAttachments(AttachmentHeaderList attachmentHeaderList, AttachmentContentList attachmentContentList) {
        List<MTOMAttachment> attachments = new ArrayList<>();
        if (attachmentHeaderList == null || attachmentContentList == null)
            return attachments;
        List<AttachmentHeaderType> headerTypes = attachmentHeaderList.getAttachmentHeader();
        List<AttachmentContentType> contentTypes = attachmentContentList.getAttachmentContent();
        for (AttachmentHeaderType headerType : headerTypes) {
            for (AttachmentContentType contentType : contentTypes) {
                if (headerType.getContentId().equals(contentType.getId())) {
                    attachments.add(new MTOMAttachment(headerType.getMimeType(), headerType.getSignaturePKCS7(), headerType.getContentId(), contentType.getContent()));
                }
            }
        }
        return attachments;
    }

    private List<LargeAttachment> createLargeAttachments(FSAttachmentsList fsAttachmentsList, RefAttachmentHeaderList refAttachmentHeaderList) throws URISyntaxException, MalformedURLException {
        List<LargeAttachment> attachments = new ArrayList<>();
        if (fsAttachmentsList == null || refAttachmentHeaderList == null)
            return attachments;
        List<FSAuthInfo> fsAuthInfos = fsAttachmentsList.getFSAttachment();
        List<RefAttachmentHeaderType> refAttachmentHeaders = refAttachmentHeaderList.getRefAttachmentHeader();
        for (int i = 0; i < fsAuthInfos.size(); i++) {
            FSAuthInfo fsAuthInfo = fsAuthInfos.get(i);
            RefAttachmentHeaderType refAttachmentHeaderType = refAttachmentHeaders.get(i);
            if (fsAuthInfo.getUuid().equals(refAttachmentHeaderType.getUuid())) {
                attachments.add(createLargeAttachment(refAttachmentHeaderType, fsAuthInfo));
            } else {
                throw new IllegalStateException("Wrong order!");
            }
        }
        return attachments;
    }

    private LargeAttachment createLargeAttachment(RefAttachmentHeaderType refAttachmentHeaderType, FSAuthInfo fsAuthInfo) throws URISyntaxException, MalformedURLException {
        String mimeType = refAttachmentHeaderType.getMimeType();
        byte[] signaturePKCS7 = refAttachmentHeaderType.getSignaturePKCS7();
        UUID uuid = UUID.fromString(refAttachmentHeaderType.getUuid());
        URL url = new URL(fsAuthInfo.getFileName());
        URI contentRef = new URI(url.getProtocol(), url.getUserInfo(), url.getHost(), url.getPort(), url.getPath(), url.getQuery(), url.getRef());
        byte[] hash = refAttachmentHeaderType.getHash().getBytes();
        LargeAttachment.TransportDetails transportDetails = new LargeAttachment.TransportDetails(fsAuthInfo.getUserName(), fsAuthInfo.getPassword());
        return new LargeAttachment(mimeType, signaturePKCS7, uuid, contentRef, hash, transportDetails);
    }

    private SMEVMetadata createSMEVMetadata(SenderProvidedRequestData request) {
        SMEVMetadata.MessageIdentity messageIdentity = new SMEVMetadata.MessageIdentity(request.getMessageID(), request.getReferenceMessageID(), request.getTransactionCode());
        ProcessingInformation processingInformation = new RequestInformation(request.getEOL(), request.getNodeID(), request.getTestMessage() != null);
        return new SMEVMetadata(messageIdentity, processingInformation);
    }

    @Override
    public GetRequestRequest createGetRequestRequest(QueryInformation query) {
        GetRequestRequest getRequestRequest = typesFactory.createGetRequestRequest();

        MessageTypeSelector messageTypeSelector = basicFactory.createMessageTypeSelector();
        messageTypeSelector.setId(SIGNATURE_ID_CALLER);
        messageTypeSelector.setTimestamp(Utils.getNow());
        messageTypeSelector.setNamespaceURI(query.getNamespaceURI());
        messageTypeSelector.setRootElementLocalName(query.getRootElementLocalName());
        messageTypeSelector.setNodeID(query.getNodeId());
        getRequestRequest.setMessageTypeSelector(messageTypeSelector);

        return getRequestRequest;
    }

    @Override
    public SMEVMetadata convertSendRequestResponse(SendRequestResponse sendRequestResponse) {
        SMEVMetadata smevMetadata = new SMEVMetadata(new SMEVMetadata.MessageIdentity(sendRequestResponse.getMessageMetadata().getMessageId(), null, null), null, null);
        smevMetadata.setSmevContext(createContext(sendRequestResponse.getMessageMetadata(), null, null, null));
        return smevMetadata;
    }

    private SMEVContext createContext(MessageMetadata metadata, String replyTo, String processingDetails, SMEVException ex) {
        SMEVContext.Participant sender = new SMEVContext.Participant(metadata.getSender()!=null?metadata.getSender().getMnemonic():"Not Detected", "Not Detected");
        SMEVContext.Participant recipient = null;
        if (metadata.getRecipient() != null) { //может быть нулевым для рассылок
            recipient = new SMEVContext.Participant(metadata.getRecipient().getMnemonic(), "Not Detected");
        }
        String destinationName = metadata.getRecipient()!=null?metadata.getRecipient().getMnemonic():"Not detected";
        String contentTypeName = "Not Detected";
        XMLGregorianCalendar sendingTimestamp = metadata.getSendingTimestamp();
        XMLGregorianCalendar deliveryTimestamp = metadata.getDeliveryTimestamp();
        String interactionType = "Not Detected";
        SMEVStatus smevStatus = convertStatus(metadata.getStatus());
        return new SMEVContext(sender, recipient, destinationName, contentTypeName, sendingTimestamp, deliveryTimestamp, processingDetails, interactionType, smevStatus, replyTo, ex);
    }

    private SMEVStatus convertStatus(InteractionStatusType interactionStatusType) {
        if (interactionStatusType == null) {
            return SMEVStatus.UNKNOWN;
        }
        SMEVStatus status = SMEVStatus.mapping.get(interactionStatusType.value());
        if (status == null) {
            return SMEVStatus.UNKNOWN;
        } else {
            return status;
        }
    }

    @Override
    public SendRequestRequest createSendRequestRequest(SMEVMessage smevMessage) {
        SendRequestRequest sendRequestRequest = typesFactory.createSendRequestRequest();
        sendRequestRequest.setAttachmentContentList(convertMTOMAttachments(smevMessage.getData().getAttachments()));
        sendRequestRequest.setSenderProvidedRequestData(createSenderProvidedRequestData(smevMessage));
        return sendRequestRequest;
    }

    private SenderProvidedResponseData createSenderProvidedResponseData(SMEVMessage smevMessage) {
        SenderProvidedResponseData senderProvidedResponseData = typesFactory.createSenderProvidedResponseData();
        senderProvidedResponseData.setId(SIGNATURE_ID_CALLER);
        senderProvidedResponseData.setMessageID(smevMessage.getSMEVMetadata().getMessageIdentity().getMessageId());
        senderProvidedResponseData.setTo(((ResponseInformation) smevMessage.getSMEVMetadata().getProcessingInformation()).getReplyTo());
        //check for null
        smevMessage.getData().accept(new ResponseBusinessMetadataParser(smevMessage.getData(), senderProvidedResponseData));
        return senderProvidedResponseData;
    }

    private SenderProvidedRequestData createSenderProvidedRequestData(SMEVMessage smevMessage) {
        SenderProvidedRequestData senderProvidedRequestData = typesFactory.createSenderProvidedRequestData();
        senderProvidedRequestData.setId(SIGNATURE_ID_CALLER);
        senderProvidedRequestData.setMessageID(smevMessage.getSMEVMetadata().getMessageIdentity().getMessageId());
        senderProvidedRequestData.setEOL(((RequestInformation) smevMessage.getSMEVMetadata().getProcessingInformation()).getEol());
        senderProvidedRequestData.setNodeID(smevMessage.getSMEVMetadata().getProcessingInformation().getNodeId());
        senderProvidedRequestData.setReferenceMessageID(smevMessage.getSMEVMetadata().getMessageIdentity().getReferenceMessageId());
        senderProvidedRequestData.setTransactionCode(smevMessage.getSMEVMetadata().getMessageIdentity().getTransactionCode());
        senderProvidedRequestData.setTestMessage(createTestMessage(smevMessage));

        smevMessage.getData().accept(new RequestBusinessMetadataParser(smevMessage.getData(), senderProvidedRequestData));

        return senderProvidedRequestData;
    }

    private void parseRequestBusinessData(BusinessContent data, SenderProvidedRequestData senderProvidedRequestData) {

    }

    private Void createTestMessage(SMEVMessage smevMessage) {
        if (smevMessage.getSMEVMetadata().getProcessingInformation() instanceof RequestInformation) {
            RequestInformation requestInformation = (RequestInformation) smevMessage.getSMEVMetadata().getProcessingInformation();
            if (requestInformation.isTestMessage()) {
                return basicFactory.createVoid();
            } else {
                return null;
            }
        } else {
            throw new IllegalArgumentException();
        }
    }


    private AttachmentContentList convertMTOMAttachments(List<SMEVAttachment> attachments) {
        if (attachments == null || attachments.isEmpty()) {
            return null;
        }
        AttachmentContentList attachmentContentList = basicFactory.createAttachmentContentList();
        for (SMEVAttachment smevAttachment : attachments) {
            if (smevAttachment instanceof MTOMAttachment) {
                MTOMAttachment mtomAttachment = (MTOMAttachment) smevAttachment;
                AttachmentContentType attachmentContentType = basicFactory.createAttachmentContentType();
                attachmentContentType.setId(mtomAttachment.getAttachmentId());
                attachmentContentType.setContent(mtomAttachment.getContent());
                attachmentContentList.getAttachmentContent().add(attachmentContentType);
            }
        }
        return attachmentContentList.getAttachmentContent() == null || attachmentContentList.getAttachmentContent().size() == 0 ? null : attachmentContentList;
    }
}
