package ru.voskhod.smev.client.api.message.impl.parsers;

import org.w3c.dom.Element;
import ru.voskhod.smev.client.api.types.message.attachment.LargeAttachment;
import ru.voskhod.smev.client.api.types.message.attachment.MTOMAttachment;
import ru.voskhod.smev.client.api.types.message.attachment.SMEVAttachment;
import ru.voskhod.smev.server.autogenerated.types.basic.*;

import java.util.List;


/**
 * The type Business data parser.
 */
public abstract class BusinessDataParser {
    /**
     * The constant typesFactory.
     */
    protected static final ru.voskhod.smev.server.autogenerated.types.ObjectFactory typesFactory = new ru.voskhod.smev.server.autogenerated.types.ObjectFactory();
    /**
     * The constant basicFactory.
     */
    protected static final ObjectFactory basicFactory = new ObjectFactory();

    /**
     * Create xmld sig signature type xmld sig signature type.
     *
     * @param element the element
     * @return the xmld sig signature type
     */
    protected XMLDSigSignatureType createXmldSigSignatureType(Element element) {
        if (element != null) {
            XMLDSigSignatureType xmldSigSignatureType = basicFactory.createXMLDSigSignatureType();
            xmldSigSignatureType.setAny(element);
            return xmldSigSignatureType;
        } else {
            return null;
        }
    }

    /**
     * Create message primary content message primary content.
     *
     * @param element the element
     * @return the message primary content
     */
    protected MessagePrimaryContent createMessagePrimaryContent(Element element) {
        MessagePrimaryContent messagePrimaryContent = basicFactory.createMessagePrimaryContent();
        messagePrimaryContent.setAny(element);
        return messagePrimaryContent;
    }

    /**
     * Create ref attachment header list ref attachment header list.
     *
     * @param attachments the attachments
     * @return the ref attachment header list
     */
    protected RefAttachmentHeaderList createRefAttachmentHeaderList(List<SMEVAttachment> attachments) {
        if (attachments == null || attachments.isEmpty()) {
            return null;
        }
        RefAttachmentHeaderList refAttachmentHeaderList = basicFactory.createRefAttachmentHeaderList();
        for (SMEVAttachment attachment : attachments) {
            if (attachment instanceof LargeAttachment) {
                LargeAttachment largeAttachment = (LargeAttachment) attachment;
                RefAttachmentHeaderType refAttachmentHeaderType = basicFactory.createRefAttachmentHeaderType();
                refAttachmentHeaderType.setUuid(largeAttachment.getUuid().toString());
                refAttachmentHeaderType.setHash(new String(largeAttachment.getCheckSum()));
                refAttachmentHeaderType.setMimeType(largeAttachment.getMimeType());
                refAttachmentHeaderType.setSignaturePKCS7(largeAttachment.getSignaturePKCS7());

                refAttachmentHeaderList.getRefAttachmentHeader().add(refAttachmentHeaderType);
            }
        }
        return refAttachmentHeaderList.getRefAttachmentHeader().size()>0?refAttachmentHeaderList:null;
    }

    /**
     * Create attachment header list attachment header list.
     *
     * @param attachments the attachments
     * @return the attachment header list
     */
    protected AttachmentHeaderList createAttachmentHeaderList(List<SMEVAttachment> attachments) {
        if (attachments == null || attachments.isEmpty()) {
            return null;
        }
        AttachmentHeaderList attachmentHeaderList = basicFactory.createAttachmentHeaderList();
        for (SMEVAttachment attachment : attachments) {
            if (attachment instanceof MTOMAttachment) {
                MTOMAttachment mtomAttachment = (MTOMAttachment) attachment;
                AttachmentHeaderType attachmentHeaderType = basicFactory.createAttachmentHeaderType();
                attachmentHeaderType.setContentId(mtomAttachment.getAttachmentId());
                attachmentHeaderType.setMimeType(mtomAttachment.getMimeType());
                attachmentHeaderType.setSignaturePKCS7(mtomAttachment.getSignaturePKCS7());
                attachmentHeaderList.getAttachmentHeader().add(attachmentHeaderType);
            }
        }
        return attachmentHeaderList.getAttachmentHeader().size()>0?attachmentHeaderList:null;
    }
}
