package ru.voskhod.smev.client.api.message.impl.parsers;

import ru.voskhod.smev.client.api.types.message.business.data.*;
import ru.voskhod.smev.client.api.types.message.business.data.request.CancelRequestContent;
import ru.voskhod.smev.client.api.types.message.business.data.request.RequestContent;
import ru.voskhod.smev.client.api.types.message.business.data.response.RejectResponseContent;
import ru.voskhod.smev.client.api.types.message.business.data.response.ResponseContent;
import ru.voskhod.smev.client.api.types.message.business.data.response.StatusResponseContent;
import ru.voskhod.smev.client.api.types.message.business.parsers.ParseContentVisitor;
import ru.voskhod.smev.server.autogenerated.types.SenderProvidedRequestData;

/**
 * The type Request business metadata parser.
 */
public class RequestBusinessMetadataParser extends BusinessDataParser implements ParseContentVisitor {

    /**
     * The Data.
     */
    BusinessContent data;
    /**
     * The Sender provided request data.
     */
    SenderProvidedRequestData senderProvidedRequestData;

    /**
     * Instantiates a new Request business metadata parser.
     *
     * @param data                      the data
     * @param senderProvidedRequestData the sender provided request data
     */
    public RequestBusinessMetadataParser(BusinessContent data, SenderProvidedRequestData senderProvidedRequestData) {
        this.data = data;
        this.senderProvidedRequestData = senderProvidedRequestData;
    }

    @Override
    public void visit(ResponseContent visitor) {

    }

    @Override
    public void visit(StatusResponseContent visitor) {

    }

    @Override
    public void visit(RejectResponseContent visitor) {

    }

    @Override
    public void visit(BusinessContent businessContent) {

    }

    @Override
    public void visit(RequestContent businessMetadata) {
        parseRequest();
    }

    @Override
    public void visit(CancelRequestContent businessMetadata) {
        //todo:
    }

    private SenderProvidedRequestData.BusinessProcessMetadata createBusinessProcessMetadata(BusinessContent data) {
        SenderProvidedRequestData.BusinessProcessMetadata businessProcessMetadata = typesFactory.createSenderProvidedRequestDataBusinessProcessMetadata();
        if(data!=null&& data.getBusinessProcessMetadata()!=null) {
            businessProcessMetadata.getAny().addAll(data.getBusinessProcessMetadata());
        }
        return businessProcessMetadata;
    }

    /**
     * Parse request.
     */
    void parseRequest() {
        senderProvidedRequestData.setBusinessProcessMetadata(createBusinessProcessMetadata(data));
        senderProvidedRequestData.setPersonalSignature(createXmldSigSignatureType(data.getPersonalSignature()));
        senderProvidedRequestData.setMessagePrimaryContent(createMessagePrimaryContent(data.getContent()));

        senderProvidedRequestData.setAttachmentHeaderList(createAttachmentHeaderList(data.getAttachments()));
        senderProvidedRequestData.setRefAttachmentHeaderList(createRefAttachmentHeaderList(data.getAttachments()));
    }
}
