package ru.voskhod.smev.client.api.message.impl.parsers;

import ru.voskhod.smev.client.api.types.message.business.data.*;
import ru.voskhod.smev.client.api.types.message.business.data.request.CancelRequestContent;
import ru.voskhod.smev.client.api.types.message.business.data.request.RequestContent;
import ru.voskhod.smev.client.api.types.message.business.data.response.RejectResponseContent;
import ru.voskhod.smev.client.api.types.message.business.data.response.ResponseContent;
import ru.voskhod.smev.client.api.types.message.business.data.response.StatusResponseContent;
import ru.voskhod.smev.client.api.types.message.business.parsers.ParseContentVisitor;
import ru.voskhod.smev.server.autogenerated.types.RejectCode;
import ru.voskhod.smev.server.autogenerated.types.SenderProvidedResponseData;

/**
 * The type Response business metadata parser.
 */
public class ResponseBusinessMetadataParser extends BusinessDataParser implements ParseContentVisitor {

    /**
     * The Smev message.
     */
    BusinessContent smevMessage;
    /**
     * The Sender provided response data.
     */
    SenderProvidedResponseData senderProvidedResponseData;

    /**
     * Instantiates a new Response business metadata parser.
     *
     * @param smevMessage                the smev message
     * @param senderProvidedResponseData the sender provided response data
     */
    public ResponseBusinessMetadataParser(BusinessContent smevMessage, SenderProvidedResponseData senderProvidedResponseData) {
        this.smevMessage = smevMessage;
        this.senderProvidedResponseData = senderProvidedResponseData;
    }

    @Override
    public void visit(ResponseContent visitor) {
        parseResponse(smevMessage, visitor, senderProvidedResponseData);
    }

    @Override
    public void visit(StatusResponseContent visitor) {
        parseStatus(smevMessage, visitor, senderProvidedResponseData);
    }

    @Override
    public void visit(RejectResponseContent visitor) {
        parseReject(smevMessage, visitor, senderProvidedResponseData);
    }

    @Override
    public void visit(BusinessContent businessContent) {

    }

    @Override
    public void visit(RequestContent businessMetadata) {

    }

    @Override
    public void visit(CancelRequestContent businessMetadata) {

    }


    private void parseResponse(BusinessContent data, ResponseContent meta, SenderProvidedResponseData senderProvidedResponseData) {
        senderProvidedResponseData.setMessagePrimaryContent(createMessagePrimaryContent(data.getContent()));
        senderProvidedResponseData.setPersonalSignature(createXmldSigSignatureType(data.getPersonalSignature()));
        senderProvidedResponseData.setAttachmentHeaderList(createAttachmentHeaderList(data.getAttachments()));
        senderProvidedResponseData.setRefAttachmentHeaderList(createRefAttachmentHeaderList(data.getAttachments()));
    }

    private void parseStatus(BusinessContent data, StatusResponseContent meta, SenderProvidedResponseData senderProvidedResponseData) {
        senderProvidedResponseData.setRequestStatus(createRequestStatus(meta));
    }

    private void parseReject(BusinessContent data, RejectResponseContent meta, SenderProvidedResponseData senderProvidedResponseData) {
        createRequestRejectStatus(senderProvidedResponseData, meta);
    }

    private SenderProvidedResponseData.RequestStatus createRequestStatus(final StatusResponseContent businessMetadata) {
        SenderProvidedResponseData.RequestStatus requestStatus = typesFactory.createSenderProvidedResponseDataRequestStatus();
        requestStatus.setStatusCode(businessMetadata.getStatusCode());
        requestStatus.setStatusDescription(businessMetadata.getStatusDescription());
        if (businessMetadata.getParameters() != null) {
            for (final String keyForValue : businessMetadata.getParameters().keySet()) {
                requestStatus.getStatusParameter().add(new SenderProvidedResponseData.RequestStatus.StatusParameter() {{
                    setKey(keyForValue);
                    setValue(businessMetadata.getParameters().get(keyForValue));
                }});
            }
        }
        return requestStatus;

    }

    private static void createRequestRejectStatus(SenderProvidedResponseData senderProvidedResponseData,
                                                  RejectResponseContent businessMetadata) {
        for (RejectResponseContent.Reject reject : businessMetadata.getRejects()) {
            final SenderProvidedResponseData.RequestRejected rejected =
                    typesFactory.createSenderProvidedResponseDataRequestRejected();
            rejected.setRejectionReasonCode(getValue(reject.getStatusCode()));
            rejected.setRejectionReasonDescription(reject.getStatusDescription());
            senderProvidedResponseData.getRequestRejected().add(rejected);
        }
    }

    private static RejectCode getValue(RejectResponseContent.RejectCode code) {
        try {
            return RejectCode.fromValue(code.value());
        } catch (Exception ex) {
            return RejectCode.UNKNOWN_REQUEST_DESCRIPTION;
        }
    }
}
