package ru.voskhod.smev.client.api.poll.impl;

import org.w3c.dom.Element;
import ru.voskhod.smev.client.api.factory.Factory;
import ru.voskhod.smev.client.api.services.poll.*;
import ru.voskhod.smev.client.api.types.exception.SMEVException;
import ru.voskhod.smev.client.api.types.exception.SMEVRuntimeException;

import javax.xml.namespace.QName;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.TimeoutException;


/**
 * The type Sync adapter.
 *
 * @param <SMEVSyncMessage> the type parameter
 * @param <SMEVMeta>        the type parameter
 */
@SuppressWarnings("WeakerAccess")
public class PollSyncAdapter<SMEVSyncMessage extends SyncMessage, SMEVMeta extends SyncMeta>
        implements SyncAdapter<SMEVSyncMessage, SMEVMeta>, SyncManager<SMEVSyncMessage, SMEVMeta> {

    private SyncTransport<SMEVSyncMessage, SMEVMeta> endpoint;
    private SyncAdapterConfig adapterConfig;
    private final Object adapterLock = new Object();
    private PollAdapter<SMEVSyncMessage> adapter = null;
    private static PollSyncAdapter instance;

    private PollSyncAdapter(SyncTransport<SMEVSyncMessage, SMEVMeta> endpoint, SyncAdapterConfig adapterConfig) {
        this.endpoint = endpoint;
        this.adapterConfig = adapterConfig;
    }

    public static SyncManager getSyncManager() {
        if (instance != null)   //todo:sync
            return instance;
        else
            throw new IllegalStateException("Not initialized");
    }


    private PollAdapter<SMEVSyncMessage> getAdapter() {
        synchronized (adapterLock) {
            if (adapter == null) {
                adapter = PollAdapter.createAdapter(new PollInterface<SMEVSyncMessage>() {

                    @Override
                    public SMEVSyncMessage getResponse(QName type, String nodeId) throws Exception {
                        return endpoint.getResponse(type, nodeId);
                    }

                    @Override
                    public String getMessageId(SMEVSyncMessage response) {
                        return getRequestMessageSMEVId(response);
                    }
                }, adapterConfig);
            }
            return adapter;
        }
    }

    @Override
    public SMEVSyncMessage sendRequestSync(final SMEVSyncMessage request, SyncCallback<SMEVSyncMessage> callback)
            throws SMEVException {

        Element content = request.getContent();
        QName type = new QName(content.getNamespaceURI(), content.getLocalName());
        try {
            Long msToWait;
            if (request.getEol() != null) {
                msToWait = request.getEol().getTime() - System.currentTimeMillis();
            } else {
                msToWait = null;
            }
            try {
                return getAdapter().syncRequest(request.getMessageId(), type, request.getNodeId(), msToWait, TimeUnit.MILLISECONDS, new Callable<Object>() {
                    @Override
                    public Object call() throws Exception {
                        // todo: multiple tries (only WebServiceException counts?)
                        endpoint.sendRequest(request);
                        return null;
                    }
                }, callback);
            } catch (InterruptedException | TimeoutException e) {
                throw new SMEVRuntimeException(e.getMessage(), e);
            }
        } catch (ExecutionException ex) {
            Throwable cause = ex.getCause();
            throw Factory.getExceptionMapperInstance(null).convert(cause);   //todo:   verify
        }
    }

    private String getRequestMessageSMEVId(SMEVSyncMessage responseResponse) {

        return responseResponse.getOriginalMessageId();

    }

    @Override
    public void shutdown() {
        synchronized (adapterLock) {
            if (adapter != null) {
                adapter.shutdownNow();
                adapter = null;
            }
        }
    }

    @SuppressWarnings("unchecked")
    @Override
    public SyncAdapter<SMEVSyncMessage, SMEVMeta> getSyncAdapter() {    //todo:sync
        if (instance != null)
            return instance;
        else
            throw new IllegalStateException("Not initialized");
    }

    @Override
    public void ack(final String messageId)
            throws SMEVException {

        endpoint.ack(messageId, true);      //todo: make it async?


    }

    @SuppressWarnings("unchecked")
    public synchronized static void init(SyncTransport endpoint, SyncAdapterConfig adapterConfig) {
        if (instance == null) {
            instance = new PollSyncAdapter(endpoint, adapterConfig);
        } else {
            instance.shutdown();
            instance = new PollSyncAdapter(endpoint, adapterConfig);
        }
    }
}
