package ru.voskhod.smev.client.api.poll.impl;

import java.util.concurrent.Callable;
import java.util.concurrent.FutureTask;

/**
 * The type Response task.
 *
 * @param <T> the type parameter
 */
final class ResponseTask<T> extends FutureTask<T> {

    private static final Callable<Object> FAKE = new Callable<Object>() {
        @Override
        public Object call() throws Exception {
            throw new IllegalStateException();
        }
    };

    /**
     * The Timestamp.
     */
    final long timestamp = System.currentTimeMillis();
    /**
     * The Message id.
     */
    final String messageId;
    /**
     * The Key.
     */
    final PollKey<T> key;

    /**
     * Instantiates a new Response task.
     *
     * @param messageId the message id
     * @param key       the key
     */
    ResponseTask(String messageId, PollKey<T> key) {
        super(ResponseTask.<T>fake());
        this.messageId = messageId;
        this.key = key;
    }

    @SuppressWarnings("unchecked")
    private static <T> Callable<T> fake() {
        return (Callable<T>) FAKE;
    }

    /**
     * Sets result.
     *
     * @param response the response
     */
    void setResult(T response) {
        set(response);
    }

    /**
     * Sets error.
     *
     * @param ex the ex
     */
    void setError(Throwable ex) {
        setException(ex);
    }
}
