package ru.voskhod.smev.client.api.poll.impl;


import ru.voskhod.smev.client.api.services.poll.SyncAdapterConfig;

import java.util.concurrent.TimeUnit;

/**
 * The type Sync adapter config.
 */
public class SyncAdapterConfigImpl implements SyncAdapterConfig {

    private long checkInterval = 10L;
    private TimeUnit checkIntervalUnit = TimeUnit.SECONDS;
    private long maxTimeout = 30;
    private TimeUnit maxTimeoutUnit = TimeUnit.MINUTES;
    private int requestThreads = 1;
    private int pollThreads = 1;

    /**
     * Sets check interval.
     *
     * @param checkInterval     the check interval
     * @param checkIntervalUnit the check interval unit
     * @return the check interval
     */
    @Override
    public SyncAdapterConfigImpl setCheckInterval(long checkInterval, TimeUnit checkIntervalUnit) {
        this.checkInterval = checkInterval;
        this.checkIntervalUnit = checkIntervalUnit;
        return this;
    }

    /**
     * Sets max timeout.
     *
     * @param maxTimeout     the max timeout
     * @param maxTimeoutUnit the max timeout unit
     * @return the max timeout
     */
    @Override
    public SyncAdapterConfig setMaxTimeout(long maxTimeout, TimeUnit maxTimeoutUnit) {
        this.maxTimeout = maxTimeout;
        this.maxTimeoutUnit = maxTimeoutUnit;
        return this;
    }

    /**
     * Sets request threads.
     *
     * @param threads the threads
     * @return the request threads
     */
    @Override
    public SyncAdapterConfigImpl setRequestThreads(int threads) {
        this.requestThreads = threads;
        return this;
    }

    /**
     * Sets poll threads.
     *
     * @param threads the threads
     * @return the poll threads
     */
    @Override
    public SyncAdapterConfigImpl setPollThreads(int threads) {
        this.pollThreads = threads;
        return this;
    }
    @Override
    public long getCheckInterval() {
        return checkInterval;
    }
    @Override
    public TimeUnit getCheckIntervalUnit() {
        return checkIntervalUnit;
    }
    @Override
    public long getMaxTimeout() {
        return maxTimeout;
    }
    @Override
    public TimeUnit getMaxTimeoutUnit() {
        return maxTimeoutUnit;
    }
    @Override
    public int getRequestThreads() {
        return requestThreads;
    }
    @Override
    public int getPollThreads() {
        return pollThreads;
    }
}
