package ru.voskhod.smev.message_exchange_service_client.push;

import org.eclipse.jetty.util.resource.Resource;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.net.URLStreamHandler;
import java.nio.channels.ReadableByteChannel;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.HashMap;
import java.util.Map;

final class DynamicResource extends Resource {

    private final Path path;
    private final Map<Path, byte[]> resources;

    DynamicResource(Map<String, byte[]> resources) {
        this.path = Paths.get(".");
        this.resources = new HashMap<>(resources.size());
        for (Map.Entry<String, byte[]> entry : resources.entrySet()) {
            this.resources.put(Paths.get(entry.getKey()), entry.getValue());
        }
    }

    DynamicResource(Path path, Map<Path, byte[]> resources) {
        this.path = path;
        this.resources = resources;
    }

    @Override
    public boolean isContainedIn(Resource r) {
        return false;
    }

    @Override
    public void close() {
    }

    @Override
    public boolean exists() {
        return resources.containsKey(path);
    }

    @Override
    public boolean isDirectory() {
        return false;
    }

    @Override
    public long lastModified() {
        return 0;
    }

    @Override
    public long length() {
        throw new UnsupportedOperationException();
    }

    @Override
    public URL getURL() {
        byte[] data = resources.get(path);
        if (data == null)
            return null;
        try {
            return new URL("file", null, -1, path.toString(), new MemURLStreamHandler(data));
        } catch (MalformedURLException ex) {
            throw new RuntimeException(ex);
        }
    }

    @Override
    public File getFile() {
        return null;
    }

    @Override
    public String getName() {
        return path.getFileName().toString();
    }

    @Override
    public InputStream getInputStream() {
        throw new UnsupportedOperationException();
    }

    @Override
    public ReadableByteChannel getReadableByteChannel() {
        throw new UnsupportedOperationException();
    }

    @Override
    public boolean delete() throws SecurityException {
        return false;
    }

    @Override
    public boolean renameTo(Resource dest) throws SecurityException {
        return false;
    }

    @Override
    public String[] list() {
        return new String[0];
    }

    @Override
    public Resource addPath(String path) {
        return new DynamicResource(this.path.resolve(path), resources);
    }

    private static final class MemURLConnection extends URLConnection {

        private final byte[] data;

        MemURLConnection(URL u, byte[] data) {
            super(u);
            this.data = data;
        }

        @Override
        public InputStream getInputStream() {
            return new ByteArrayInputStream(data);
        }

        @Override
        public void connect() {
        }
    }

    private static final class MemURLStreamHandler extends URLStreamHandler {

        private final byte[] data;

        private MemURLStreamHandler(byte[] data) {
            this.data = data;
        }

        @Override
        protected URLConnection openConnection(URL u) {
            return new MemURLConnection(u, data);
        }
    }
}
