package ru.voskhod.smev.client.api.template.impl;

import org.apache.commons.codec.binary.Base64;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import ru.voskhod.smev.client.api.services.identification.IdentityService;
import ru.voskhod.smev.client.api.services.signature.Signer;
import ru.voskhod.smev.client.api.services.template.LargeAttachmentTemplate;
import ru.voskhod.smev.client.api.services.template.configuration.LargeAttachmentTemplateConfiguration;
import ru.voskhod.smev.client.api.services.transport.LargeAttachmentTransport;
import ru.voskhod.smev.client.api.types.exception.SMEVException;
import ru.voskhod.smev.client.api.types.exception.SMEVRuntimeException;
import ru.voskhod.smev.client.api.types.exception.processing.SMEVSignatureException;
import ru.voskhod.smev.client.api.types.message.attachment.LargeAttachment;
import ru.voskhod.smev.client.api.types.message.attachment.SMEVAttachment;
import ru.voskhod.smev.client.api.util.Utils;
import sun.reflect.generics.reflectiveObjects.NotImplementedException;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.UUID;


/**
 * The type Large attachment template.
 */
public class LargeAttachmentTemplateImpl implements LargeAttachmentTemplate {
    private static final Logger logger = LoggerFactory.getLogger(LargeAttachmentTemplateImpl.class);
    private Signer signer;
    private IdentityService identificationService;
    private LargeAttachmentTransport largeAttachmentTransport; //todo:

    /**
     * Instantiates a new Large attachment template.
     *
     * @param signer                   the signer
     * @param identificationService    the identification service
     * @param largeAttachmentTransport the large attachment transport
     */
    public LargeAttachmentTemplateImpl(Signer signer, IdentityService identificationService, LargeAttachmentTransport largeAttachmentTransport) {
        this.signer = signer;
        this.identificationService = identificationService;
        this.largeAttachmentTransport = largeAttachmentTransport;
    }

    /**
     * Instantiates a new Large attachment template.
     *
     * @param config the config
     */
    public LargeAttachmentTemplateImpl(LargeAttachmentTemplateConfiguration config) {
        throw new NotImplementedException();
        //TODO
    }


    @Override
    public List<LargeAttachment> createAttachments(List<File> files) throws SMEVSignatureException, SMEVException {
        return createAttachments(files, createCheckSum(files));
    }

    @Override
    public List<LargeAttachment> createAttachments(List<File> files, List<byte[]> checkSums) throws SMEVSignatureException, SMEVException {
        return createAttachments(files, checkSums, sign(checkSums));
    }

    @Override
    public List<LargeAttachment> createAttachments(List<File> files, List<byte[]> checkSums, List<byte[]> personalSignatures) throws SMEVException {
        List<LargeAttachment> attachments = new ArrayList<>();
        if (files == null) {
            return attachments;
        } else if (checkSums != null && personalSignatures != null) {
            if (files.size() != checkSums.size() && (files.size() != personalSignatures.size())) {
                throw new SMEVRuntimeException("Неккоректные данные для приложений: величины массивов не совпадают");///TODO:exception
            }
        }
        logger.debug("Генерируем приложения");
        for (int i = 0; i < files.size(); i++) {
            File file = files.get(i);
            byte[] checkSum = checkSums.get(i);
            byte[] personalSignature = null;
            if (personalSignatures != null && !personalSignatures.isEmpty()) {
                personalSignature = personalSignatures.get(i);
            }
            attachments.add(i, createAttachment(file, checkSum, personalSignature));
        }
        return attachments;
    }

    private LargeAttachment createAttachment(File file, byte[] checkSum, byte[] personalSignature) throws SMEVException {
        String mimeType = Utils.guessMimeType(file);
        UUID uuid = identificationService.generateAttachmentUUID();
        LargeAttachment largeAttachment = new LargeAttachment(mimeType, personalSignature, uuid, file.toURI(), checkSum);
        return largeAttachment;
    }

    private List<byte[]> createCheckSum(List<File> files) throws SMEVSignatureException {
        List<byte[]> checkSums = new ArrayList<>();
        if (signer != null) {
            logger.debug("Генерируем дайджесты для приложений");
            for (int i = 0; i < files.size(); i++) {
                File file = files.get(i);
                byte[] digest;
                digest = signer.getDigest(file);
                checkSums.add(i, Base64.encodeBase64String(digest).getBytes());
            }
        }
        return checkSums;
    }

    private List<byte[]> sign(List<byte[]> checkSums) throws SMEVSignatureException, SMEVRuntimeException {
        List<byte[]> personalSignatures = new ArrayList<>();
        if (signer != null) {
            logger.debug("Генерируем подписи для приложений");
            for (int i = 0; i < checkSums.size(); i++) {
                byte[] checkSum = checkSums.get(i);
                personalSignatures.add(i, signer.signPKCS7Detached(Base64.decodeBase64(checkSum)));
            }
        }
        return personalSignatures;
    }
    @Override
    public void sendLargeAttachments(List<SMEVAttachment> attachments) throws SMEVRuntimeException {
        if (attachments == null || largeAttachmentTransport == null) {
            logger.debug("LargeAttachments will not be processed to deliver");
            return;
        }
        for (SMEVAttachment attachment : attachments) {
            if (attachment instanceof LargeAttachment) {
                largeAttachmentTransport.upload((LargeAttachment) attachment);
            }
        }
    }
}
