package ru.voskhod.smev.client.api.template.impl;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import ru.voskhod.smev.client.api.services.identification.IdentityService;
import ru.voskhod.smev.client.api.services.signature.Signer;
import ru.voskhod.smev.client.api.services.template.MTOMAttachmentTemplate;
import ru.voskhod.smev.client.api.services.template.configuration.MTOMAttachmentTemplateConfiguration;
import ru.voskhod.smev.client.api.types.exception.SMEVRuntimeException;
import ru.voskhod.smev.client.api.types.exception.processing.SMEVSignatureException;
import ru.voskhod.smev.client.api.types.message.attachment.MTOMAttachment;
import ru.voskhod.smev.client.api.util.Utils;
import sun.reflect.generics.reflectiveObjects.NotImplementedException;

import javax.activation.DataHandler;
import javax.activation.FileDataSource;
import java.io.File;
import java.util.ArrayList;
import java.util.List;


/**
 * The type Mtom attachment template.
 */
public class MTOMAttachmentTemplateImpl implements MTOMAttachmentTemplate {
    private static final Logger logger = LoggerFactory.getLogger(MTOMAttachmentTemplateImpl.class);
    private Signer signer;
    private IdentityService identificationService;

    /**
     * Instantiates a new Mtom attachment template.
     *
     * @param signer                the signer
     * @param identificationService the identification service
     */
    public <T extends MTOMAttachmentTemplateConfiguration> MTOMAttachmentTemplateImpl(Signer signer, IdentityService identificationService) {
        this.signer = signer;
        this.identificationService = identificationService;
    }

    /**
     * Instantiates a new Mtom attachment template.
     *
     * @param config the config
     */
    public <T extends MTOMAttachmentTemplateConfiguration> MTOMAttachmentTemplateImpl(T config) {
        throw new NotImplementedException();
    }

    @Override
    public List<MTOMAttachment> createAttachments(List<File> files) throws SMEVSignatureException, SMEVRuntimeException {
        return createAttachments(files, sign(files));
    }

    @Override
    public List<MTOMAttachment> createAttachments(List<File> files, List<byte[]> personalSignatures) {
        List<MTOMAttachment> attachments = new ArrayList<>();
        if(files==null){
            return attachments;
        }
        logger.debug("Создаем приложения");
        for (int i = 0; i < files.size(); i++) {
            File file = files.get(i);
            byte[] personalSignature = null;
            if (personalSignatures != null && !personalSignatures.isEmpty()) {
                personalSignature = personalSignatures.get(i);
            }
            attachments.add(i, createAttachment(file, personalSignature, i));
        }
        return attachments;
    }

    private MTOMAttachment createAttachment(File file, byte[] personalSignature, int attachmentNumber) {
        String mimeType = Utils.guessMimeType(file);
        String attachmentId = file.getName();//identificationService.generateAttachmentId(mimeType, attachmentNumber);
        DataHandler content = new DataHandler(new FileDataSource(file));
        return new MTOMAttachment(mimeType, personalSignature, attachmentId, content);
    }

    private List<byte[]> sign(List<File> files) throws SMEVSignatureException, SMEVRuntimeException {
        List<byte[]> personalSignatures = new ArrayList<>();
        if (signer != null) {
            logger.debug("Подписываем файлы");
            for (int i = 0; i < files.size(); i++) {
                File file = files.get(i);
                byte[] digest = signer.getDigest(file);
                personalSignatures.add(i, signer.signPKCS7Detached(digest));
            }
        }
        return personalSignatures;
    }
}
