package ru.voskhod.smev.client.api.template.loader;

import ru.voskhod.smev.client.api.factory.loaders.TemplateLoader;
import ru.voskhod.smev.client.api.services.identification.IdentityService;
import ru.voskhod.smev.client.api.services.messaging.ExceptionMapper;
import ru.voskhod.smev.client.api.services.messaging.MessageMapper;
import ru.voskhod.smev.client.api.services.signature.PostValidationCallback;
import ru.voskhod.smev.client.api.services.signature.Signer;
import ru.voskhod.smev.client.api.services.template.LargeAttachmentTemplate;
import ru.voskhod.smev.client.api.services.template.MTOMAttachmentTemplate;
import ru.voskhod.smev.client.api.services.template.WSTemplate;
import ru.voskhod.smev.client.api.services.template.configuration.*;
import ru.voskhod.smev.client.api.services.transport.GeoMessageTransport;
import ru.voskhod.smev.client.api.services.transport.LargeAttachmentTransport;
import ru.voskhod.smev.client.api.services.transport.MessageTransport;
import ru.voskhod.smev.client.api.services.transport.configuration.LargeAttachmentTransportConfiguration;
import ru.voskhod.smev.client.api.services.validation.Validator;
import ru.voskhod.smev.client.api.template.impl.*;
import ru.voskhod.smev.client.api.types.exception.SMEVRuntimeException;

/**
 * The type Loader.
 */
public class Loader implements TemplateLoader {
    @Override
    public <T extends WSTemplateConfiguration> WSTemplate getWsTemplate(T config) throws SMEVRuntimeException {
        return new WSTemplateImpl(config);
    }

    @Override
    public <T extends WSTemplateConfiguration> WSTemplate getWsTemplate(Signer signer,
                                                                        Validator validator,
                                                                        MessageMapper mapper,
                                                                        MessageTransport transport,
                                                                        ExceptionMapper manager,
                                                                        LargeAttachmentTransport atransport,
                                                                        T config) throws SMEVRuntimeException {
        return new WSTemplateImpl(signer, validator, mapper, transport, manager, atransport, config);
    }

    @Override
    public <T extends WSTemplateConfiguration> WSTemplate getWsTemplate(Signer signer,
                                                                        Validator validator,
                                                                        MessageMapper mapper,
                                                                        MessageTransport transport,
                                                                        ExceptionMapper manager,
                                                                        LargeAttachmentTransport atransport,
                                                                        T config,
                                                                        PostValidationCallback postValidationCallback) throws SMEVRuntimeException {
        return new WSTemplateImpl(signer, validator, mapper, transport, manager, atransport, config, postValidationCallback);
    }

    @Override
    public <T extends GeoTemplateConfiguration> WSTemplate getWsTemplate(Signer signer,
                                                                         Validator validator,
                                                                         MessageMapper mapper,
                                                                         GeoMessageTransport transport,
                                                                         ExceptionMapper manager,
                                                                         LargeAttachmentTransport atransport,
                                                                         T config) throws SMEVRuntimeException {
        return new WSTemplateImpl(signer, validator, mapper, transport, manager, atransport, config);
    }

    @Override
    public <T extends GeoTemplateConfiguration> WSTemplate getWsTemplate(Signer signer,
                                                                         Validator validator,
                                                                         MessageMapper mapper,
                                                                         GeoMessageTransport transport,
                                                                         ExceptionMapper manager,
                                                                         LargeAttachmentTransport atransport,
                                                                         T config,
                                                                         PostValidationCallback postValidationCallback) throws SMEVRuntimeException {
        return new WSTemplateImpl(signer, validator, mapper, transport, manager, atransport, config, postValidationCallback);
    }

    @Override
    public <T extends LargeAttachmentTemplateConfiguration> LargeAttachmentTemplate getLargeAttachmentsTemplate(T config) {
        return new LargeAttachmentTemplateImpl(config);
    }

    @Override
    public <T extends LargeAttachmentTemplateConfiguration> LargeAttachmentTemplate getLargeAttachmentsTemplate(Signer signer, IdentityService identificationService, LargeAttachmentTransport largeAttachmentTransport) {
        return new LargeAttachmentTemplateImpl(signer, identificationService, largeAttachmentTransport);
    }

    @Override
    public <T extends LargeAttachmentTransportConfiguration> LargeAttachmentTransport getLargeAttachmentTransportTemplate(T config) {
        return new LargeAttachmentTransportImpl(config);
    }

    @Override
    public <T extends MTOMAttachmentTemplateConfiguration> MTOMAttachmentTemplate getMTOMTemplate(T config) {
        return new MTOMAttachmentTemplateImpl(config);
    }

    @Override
    public <T extends MTOMAttachmentTemplateConfiguration> MTOMAttachmentTemplate getMTOMTemplate(Signer signer, IdentityService identificationService) {
        return new MTOMAttachmentTemplateImpl(signer, identificationService);
    }

    @Override
    public <T extends TemplateConfiguration> TemplateImpl getTemplate(T config) throws SMEVRuntimeException {
        return new TemplateImpl(config, config.getSignerConfig().getCertificateAlias(), config.getSignerConfig().getPrivateKeyAlias(), config.getSignerConfig().getPrivateKeyPassword());
    }

    @Override
    public <T extends GeoTemplateConfiguration> TemplateImpl getTemplate(T config) throws SMEVRuntimeException {
        return new TemplateImpl(
                config,
                config.getSignerConfig().getCertificateAlias(),
                config.getSignerConfig().getPrivateKeyAlias(),
                config.getSignerConfig().getPrivateKeyPassword()
        );
    }
}
