package ru.voskhod.smev.client.api.template.marshaller;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import ru.voskhod.smev.client.api.types.exception.SMEVRuntimeException;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMResult;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import java.io.StringWriter;

/**
 * The type Marshaller.
 */
public class Marshaller {
    /**
     * The constant SERVER_API_PACKAGE.
     */
    public static final String SERVER_API_PACKAGE = "ru.voskhod.smev.server.autogenerated.types:ru.voskhod.smev.server.autogenerated.types.basic:ru.voskhod.smev.server.autogenerated.types.fault";
    private static final ThreadLocal<JAXBContext> jaxbContext = new ThreadLocal<>();

    private static void init() { //"ru.voskhod.smev.server.autogenerated.types";
        if (jaxbContext.get() == null) {
            try {
                jaxbContext.set(JAXBContext.newInstance(SERVER_API_PACKAGE));
            } catch (JAXBException e) {
                throw new RuntimeException(e);
            }
        }
    }

    /**
     * Marshal element.
     *
     * @param obj the obj
     * @return the element
     * @throws SMEVRuntimeException the smev runtime exception
     */
    public static Element marshal(Object obj) throws SMEVRuntimeException {
        if (jaxbContext.get() == null) { //bad design
            init();
        }
        DOMResult domResult = new DOMResult();

        try {
            jaxbContext.get().createMarshaller().marshal(obj, domResult);
        } catch (JAXBException e) {
            throw new SMEVRuntimeException(e.getMessage(), e);
        }

        return ((Document) domResult.getNode()).getDocumentElement();
    }

    /**
     * Unmarshal object.
     *
     * @param element the element
     * @return the object
     * @throws SMEVRuntimeException the smev runtime exception
     */
    public static Object unmarshal(Element element) throws SMEVRuntimeException {
        if (jaxbContext.get() == null) {
            init();
        }
        try {
            return jaxbContext.get().createUnmarshaller().unmarshal(element);
        } catch (JAXBException e) {
            throw new SMEVRuntimeException(e.getMessage(), e);
        }
    }

    /**
     * Convert element to string string.
     *
     * @param rs the rs
     * @return the string
     * @throws TransformerException the transformer exception
     * @throws SMEVRuntimeException the smev runtime exception
     */
    public static String convertElementToString(Object rs) throws TransformerException, SMEVRuntimeException {
        TransformerFactory transFactory = TransformerFactory.newInstance();
        Transformer transformer = transFactory.newTransformer();
        StringWriter buffer = new StringWriter();
        transformer.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION, "false");
        transformer.transform(new DOMSource(Marshaller.marshal(rs)),
                new StreamResult(buffer));
        return buffer.toString();
    }
}
