package ru.voskhod.smev.client.api.transaction.impl;

import ru.voskhod.smev.client.api.services.transaction.TransactionService;
import ru.voskhod.smev.client.api.services.transaction.configuration.TxConfiguration;
import ru.voskhod.smev.server.autogenerated.transactionmaker.*;

import javax.xml.namespace.QName;
import javax.xml.ws.WebServiceException;
import java.net.URL;


/**
 * The type Transaction service.
 */
public class TransactionServiceImpl implements TransactionService {

    private static QName MARKERSERVICE_QNAME = new QName("http://endpoints.transactionmarker.voskhod.ru/", "MarkerService");
    private static URL MARKERSERVICE_WSDL_LOCATION;
    private static WebServiceException MARKERSERVICE_EXCEPTION;

    private static volatile boolean init = false;

    /**
     * Instantiates a new Transaction service.
     *
     * @param txConfig the tx config
     */
    public TransactionServiceImpl(TxConfiguration txConfig) {
        init(txConfig);
    }

    private static synchronized void init(TxConfiguration txConfig) {
        if (!init) {
            synchronized (TransactionServiceImpl.class) {
                if (!init) {
                    try {
                        MARKERSERVICE_WSDL_LOCATION = txConfig.getServiceUrl();
                    } catch (Exception e) {
                        MARKERSERVICE_EXCEPTION = new WebServiceException(e);
                    }
                    TransactionServiceImpl.init = true;
                }
            }
        }
    }

    @Override
    public String getTransactionCode(String frguServiceCode, String frguInteractionPartisipantCode, String frguServiceDescription, String frguServiceRecipientDescription) {
        if (MARKERSERVICE_EXCEPTION != null) {
            throw MARKERSERVICE_EXCEPTION;
        }
        IMarker port = new MarkerService(MARKERSERVICE_WSDL_LOCATION, MARKERSERVICE_QNAME).getMarkerPort();
        FRGUInformation information = new FRGUInformation();

        ObjectFactory objectFactory = new ObjectFactory();

        information.setFRGUServiceCode(frguServiceCode);
        information.setFRGUInteractionPartisipantCode(frguInteractionPartisipantCode);
        information.setFRGUServiceDescription(objectFactory.createFRGUInformationFRGUServiceDescription(frguServiceDescription));
        information.setFRGUServiceRecipientDescription(objectFactory.createFRGUInformationFRGUServiceRecipientDescription(frguServiceRecipientDescription));

        Marker marker = port.generateMark(information);

        return marker.getTransactionCode();
    }
}
