package ru.voskhod.smev.client.api.transport.impl;

import ru.voskhod.smev.client.api.services.transport.MessageTransport;
import ru.voskhod.smev.client.api.services.transport.configuration.MessageTransportConfiguration;
import ru.voskhod.smev.client.api.transport.Constants;
import ru.voskhod.smev.server.autogenerated.types.basic.Void;
import ru.voskhod.smev.server.autogenerated.types.*;

/**
 * The type Message transport.
 */
public class MessageTransportImpl implements MessageTransport<SendRequestRequest, SendRequestResponse,
        SendResponseRequest, SendResponseResponse,
        GetRequestRequest, GetRequestResponse,
        GetResponseRequest, GetResponseResponse,
        GetStatusRequest, GetStatusResponse,
        AckRequest, Void,
        Object, Object> {

    private final TransportProvider provider;

    /**
     * Instantiates a new Message transport.
     *
     * @param config the config
     * @throws Exception the exception
     */
    public <T extends MessageTransportConfiguration> MessageTransportImpl(T config)
            throws Exception {
        this.provider = new TransportProvider(config.getUrl(), Constants.EXCHANGE_MESSAGE_SERVICE, config.getTimeout());
    }

    @Override
    public SendRequestResponse sendRequest(SendRequestRequest parameters)
            throws Exception {
        return provider.port().sendRequest(parameters);
    }

    @Override
    public SendResponseResponse sendResponse(SendResponseRequest parameters)
            throws Exception {
        return provider.port().sendResponse(parameters);
    }

    @Override
    public GetRequestResponse getRequest(GetRequestRequest parameters)
            throws Exception {
        return provider.port().getRequest(parameters);
    }

    @Override
    public GetStatusResponse getStatus(GetStatusRequest parameters)
            throws Exception {
        return provider.port().getStatus(parameters);
    }

    @Override
    public GetResponseResponse getResponse(GetResponseRequest parameters)
            throws Exception {
        return provider.port().getResponse(parameters);
    }

    @Override
    public Void ack(AckRequest parameters) throws Exception {
        return provider.port().ack(parameters);
    }

    @Override
    public Object getIncomingQueueStatistics(Object parameters)
            throws Exception {
      throw new UnsupportedOperationException();
    }
}
