package ru.voskhod.smev.client.api.transport.impl;

import ru.voskhod.smev.client.api.types.exception.SMEVRuntimeException;
import ru.voskhod.smev.server.autogenerated.SMEVMessageExchangePortType;
import ru.voskhod.smev.server.autogenerated.SMEVMessageExchangeService;

import javax.xml.namespace.QName;
import javax.xml.ws.BindingProvider;
import javax.xml.ws.soap.SOAPBinding;
import java.net.URL;

final class TransportProvider {
    private final String url;
    private final QName name;
    private final int timeout;
    private volatile SMEVMessageExchangePortType port;


    TransportProvider(String url, QName name, int timeout) {
        this.url = url;
        this.name = name;
        this.timeout = timeout;
    }

    SMEVMessageExchangePortType port() throws SMEVRuntimeException {
        if (port == null) {
            synchronized (TransportProvider.class) {
                if (port == null) {
                    try {
                        port = createPort(url, name, timeout);
                    } catch (Exception e) {
                        throw new SMEVRuntimeException("Невозможно получить доступ до транспортного узла");
                    }
                }
            }
        }
        return port;
    }


    private static SMEVMessageExchangePortType createPort(String url, QName serviceQName, int timeout)
            throws Exception {

        SMEVMessageExchangeService service = new SMEVMessageExchangeService(new URL(url), serviceQName);

        SMEVMessageExchangePortType port = service.getSMEVMessageExchangeEndpoint();
        ((BindingProvider) port).getRequestContext().put("com.sun.xml.internal.ws.connect.timeout", timeout);
        ((BindingProvider) port).getRequestContext().put("com.sun.xml.internal.ws.request.timeout", timeout);
        ((BindingProvider) port).getRequestContext().put("com.sun.xml.ws.request.timeout", timeout);
        ((BindingProvider) port).getRequestContext().put("com.sun.xml.ws.connect.timeout", timeout);

        SOAPBinding binding = (SOAPBinding) ((BindingProvider) port).getBinding();
        binding.setMTOMEnabled(true);
        return port;
    }
}
