package ru.voskhod.smev.client.toolset.adminserv;

import ru.voskhod.smev.client.api.factory.Factory;
import ru.voskhod.smev.client.api.services.admin.AdminService;
import ru.voskhod.smev.client.api.types.exception.SMEVRuntimeException;
import ru.voskhod.smev.client.toolset.commons.ToolConfiguration;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.nio.charset.Charset;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.cert.CertificateException;
import java.util.Enumeration;
import java.util.HashSet;
import java.util.Set;


public final class Main {

    private static AdminService getService(ToolConfiguration config) throws SMEVRuntimeException {
        return Factory.getAdminInstance(config.getAdminServiceConfiguration());
    }

    public static void main(String... args) throws Exception {
        final ToolConfiguration config = ToolConfiguration.getDefault();
        boolean result = true;
        if (args.length > 0) {
            if ("-aliases".equalsIgnoreCase(args[0])) {
                final String providerName = config.getAdminServiceConfiguration()
                        .getSignerConfig().getProviderName();
                final Set<String> aliases = CryptoUtilities.listAliases(providerName);
                for (String alias : aliases) {
                    System.out.println(alias);
                }
            } else if ("-switch".equalsIgnoreCase(args[0])) {
                final AdminService service = getService(config);
                result = service.switchNode();
                System.out.println("Switched: " + (result ? "TRUE" : "FALSE"));
            } else if ("-current".equalsIgnoreCase(args[0])) {
                final AdminService service = getService(config);
                final String idNode = service.getCurrentNode();
                System.out.println("Node: " + idNode);
            } else if ("-help".equalsIgnoreCase(args[0])) {
                printUsage();
            } else {
                printUsage();
            }
        } else {
            printUsage();
        }
        System.exit(result ? 0 : 1);
    }

    private static void printUsage() {
        System.out.println("adminservtool [option]");
        System.out.println("-help    : выводит помощь по командам. Этот текст");
        System.out.println("-switch  : переключает основной ГРУ на резервный");
        System.out.println("-current : выводит на экран мнемонику текущего ГРУ");
    }

    private static final class CryptoUtilities {
        private static final Charset UTF8 = Charset.forName("UTF-8");
        private static final String CRYPTOPRO_DIGT_KEYSTORE = "CryptoProCSPKeyStore";
        private static final String CRYPTOPRO_JCP_KEYSTORE = "HDImageStore";
        public static final String CURRENT_USER_CONTAINERS_KS_TYPE = "CurrentUser/Containers";

        public static Set<String> listAliases(String providerName) throws KeyStoreException, CertificateException,
                NoSuchAlgorithmException, IOException, NoSuchProviderException {
            Set<String> result = new HashSet<>();
            final boolean isDigt = "DIGT".equalsIgnoreCase(providerName);
            KeyStore ks = KeyStore.getInstance(isDigt ? CRYPTOPRO_DIGT_KEYSTORE : CRYPTOPRO_JCP_KEYSTORE);
            if (isDigt) {
                try (InputStream stream = new ByteArrayInputStream(CURRENT_USER_CONTAINERS_KS_TYPE.getBytes(UTF8))) {
                    ks.load(stream, null);
                }
            } else {
                ks.load(null);
            }
            final Enumeration<String> en = ks.aliases();
            while (en.hasMoreElements()) {
                final String element = en.nextElement();
                result.add(element);
            }
            return result;
        }
    }
}
