package ru.voskhod.smev.client.api.util;

import org.apache.tika.Tika;
import org.apache.tika.mime.MimeTypes;

import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;
import java.io.File;
import java.util.Date;
import java.util.GregorianCalendar;


/**
 * The type Utils.
 */
public class Utils {
    /**
     * The constant UNKNOWN_MIME_TYPE.
     */
    public static final String UNKNOWN_MIME_TYPE = "";
    /**
     * The constant UNKNOWN_FILE_EXTENSION.
     */
    public static final String UNKNOWN_FILE_EXTENSION = "";


    private static final ThreadLocal<Tika> tika = new ThreadLocal<Tika>(){
        @Override
        public Tika initialValue(){
            return new Tika();
        }
    };
    private static final ThreadLocal<DatatypeFactory> datatypeFactory = new ThreadLocal<>();


    private Utils() {
    }

    /**
     * Gets xml timestamp.
     *
     * @param date the date
     * @return the xml timestamp
     */
    public static XMLGregorianCalendar getXmlTimestamp(Date date) {
        if (datatypeFactory.get() == null) {
            init();
        }
        GregorianCalendar gc = new GregorianCalendar();
        gc.setTime(date);
        return datatypeFactory.get().newXMLGregorianCalendar(gc);
    }

    /**
     * Gets now.
     *
     * @return the now
     */
    public static XMLGregorianCalendar getNow() {
        return getXmlTimestamp(new Date());
    }

    /**
     * Пытается определить MIME тип указанного файла
     *
     * @param file файл
     * @return MIME тип или пустую строку, если не удалось определить тип
     */
    public static String guessMimeType(File file) {
        try {
            String result = tika.get().detect(file);
            return result == null ? UNKNOWN_MIME_TYPE : result;
        } catch (Exception ex) {
            return UNKNOWN_MIME_TYPE;
        }
    }

    /**
     * Возращает наиболее часто используемое расширение файла для указанного MIME типа.
     *
     * @param mimeType MIME тип файла
     * @return расширение или пустую строку, если не удалось определить
     */
    public static String guessFileExtension(String mimeType) {
        try {
            return MimeTypes.getDefaultMimeTypes().forName(mimeType).getExtension();
        } catch (Exception ex) {
            return UNKNOWN_FILE_EXTENSION;
        }
    }

    private static void init() {

        if (datatypeFactory.get() == null) {
            try {
                datatypeFactory.set(DatatypeFactory.newInstance());
            } catch (DatatypeConfigurationException e) {
                throw new RuntimeException(e);
            }
        }
        if (tika.get() == null) {
            tika.set(new Tika());
        }
    }

}
